# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Output/OutGeoLam.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Output/OutGeoLam
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import set_array, check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

from numpy import array, array_equal
from ._check import InitUnKnowClassError


class OutGeoLam(FrozenClass):
    """Gather the geometrical and the global outputs of a lamination"""

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        name_phase=None,
        BH_curve=None,
        Ksfill=None,
        S_slot=None,
        S_slot_wind=None,
        S_wind_act=None,
        per_a=None,
        is_antiper_a=None,
        per_t=None,
        is_antiper_t=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "name_phase" in list(init_dict.keys()):
                name_phase = init_dict["name_phase"]
            if "BH_curve" in list(init_dict.keys()):
                BH_curve = init_dict["BH_curve"]
            if "Ksfill" in list(init_dict.keys()):
                Ksfill = init_dict["Ksfill"]
            if "S_slot" in list(init_dict.keys()):
                S_slot = init_dict["S_slot"]
            if "S_slot_wind" in list(init_dict.keys()):
                S_slot_wind = init_dict["S_slot_wind"]
            if "S_wind_act" in list(init_dict.keys()):
                S_wind_act = init_dict["S_wind_act"]
            if "per_a" in list(init_dict.keys()):
                per_a = init_dict["per_a"]
            if "is_antiper_a" in list(init_dict.keys()):
                is_antiper_a = init_dict["is_antiper_a"]
            if "per_t" in list(init_dict.keys()):
                per_t = init_dict["per_t"]
            if "is_antiper_t" in list(init_dict.keys()):
                is_antiper_t = init_dict["is_antiper_t"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.name_phase = name_phase
        self.BH_curve = BH_curve
        self.Ksfill = Ksfill
        self.S_slot = S_slot
        self.S_slot_wind = S_slot_wind
        self.S_wind_act = S_wind_act
        self.per_a = per_a
        self.is_antiper_a = is_antiper_a
        self.per_t = per_t
        self.is_antiper_t = is_antiper_t

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        OutGeoLam_str = ""
        if self.parent is None:
            OutGeoLam_str += "parent = None " + linesep
        else:
            OutGeoLam_str += "parent = " + str(type(self.parent)) + " object" + linesep
        OutGeoLam_str += (
            "name_phase = "
            + linesep
            + str(self.name_phase).replace(linesep, linesep + "\t")
            + linesep
        )
        OutGeoLam_str += (
            "BH_curve = "
            + linesep
            + str(self.BH_curve).replace(linesep, linesep + "\t")
            + linesep
            + linesep
        )
        OutGeoLam_str += "Ksfill = " + str(self.Ksfill) + linesep
        OutGeoLam_str += "S_slot = " + str(self.S_slot) + linesep
        OutGeoLam_str += "S_slot_wind = " + str(self.S_slot_wind) + linesep
        OutGeoLam_str += "S_wind_act = " + str(self.S_wind_act) + linesep
        OutGeoLam_str += "per_a = " + str(self.per_a) + linesep
        OutGeoLam_str += "is_antiper_a = " + str(self.is_antiper_a) + linesep
        OutGeoLam_str += "per_t = " + str(self.per_t) + linesep
        OutGeoLam_str += "is_antiper_t = " + str(self.is_antiper_t) + linesep
        return OutGeoLam_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.name_phase != self.name_phase:
            return False
        if not array_equal(other.BH_curve, self.BH_curve):
            return False
        if other.Ksfill != self.Ksfill:
            return False
        if other.S_slot != self.S_slot:
            return False
        if other.S_slot_wind != self.S_slot_wind:
            return False
        if other.S_wind_act != self.S_wind_act:
            return False
        if other.per_a != self.per_a:
            return False
        if other.is_antiper_a != self.is_antiper_a:
            return False
        if other.per_t != self.per_t:
            return False
        if other.is_antiper_t != self.is_antiper_t:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if other._name_phase != self._name_phase:
            diff_list.append(name + ".name_phase")
        if not array_equal(other.BH_curve, self.BH_curve):
            diff_list.append(name + ".BH_curve")
        if other._Ksfill != self._Ksfill:
            diff_list.append(name + ".Ksfill")
        if other._S_slot != self._S_slot:
            diff_list.append(name + ".S_slot")
        if other._S_slot_wind != self._S_slot_wind:
            diff_list.append(name + ".S_slot_wind")
        if other._S_wind_act != self._S_wind_act:
            diff_list.append(name + ".S_wind_act")
        if other._per_a != self._per_a:
            diff_list.append(name + ".per_a")
        if other._is_antiper_a != self._is_antiper_a:
            diff_list.append(name + ".is_antiper_a")
        if other._per_t != self._per_t:
            diff_list.append(name + ".per_t")
        if other._is_antiper_t != self._is_antiper_t:
            diff_list.append(name + ".is_antiper_t")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        if self.name_phase is not None:
            for value in self.name_phase:
                S += getsizeof(value)
        S += getsizeof(self.BH_curve)
        S += getsizeof(self.Ksfill)
        S += getsizeof(self.S_slot)
        S += getsizeof(self.S_slot_wind)
        S += getsizeof(self.S_wind_act)
        S += getsizeof(self.per_a)
        S += getsizeof(self.is_antiper_a)
        S += getsizeof(self.per_t)
        S += getsizeof(self.is_antiper_t)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        OutGeoLam_dict = dict()
        OutGeoLam_dict["name_phase"] = (
            self.name_phase.copy() if self.name_phase is not None else None
        )
        if self.BH_curve is None:
            OutGeoLam_dict["BH_curve"] = None
        else:
            OutGeoLam_dict["BH_curve"] = self.BH_curve.tolist()
        OutGeoLam_dict["Ksfill"] = self.Ksfill
        OutGeoLam_dict["S_slot"] = self.S_slot
        OutGeoLam_dict["S_slot_wind"] = self.S_slot_wind
        OutGeoLam_dict["S_wind_act"] = self.S_wind_act
        OutGeoLam_dict["per_a"] = self.per_a
        OutGeoLam_dict["is_antiper_a"] = self.is_antiper_a
        OutGeoLam_dict["per_t"] = self.per_t
        OutGeoLam_dict["is_antiper_t"] = self.is_antiper_t
        # The class name is added to the dict for deserialisation purpose
        OutGeoLam_dict["__class__"] = "OutGeoLam"
        return OutGeoLam_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.name_phase = None
        self.BH_curve = None
        self.Ksfill = None
        self.S_slot = None
        self.S_slot_wind = None
        self.S_wind_act = None
        self.per_a = None
        self.is_antiper_a = None
        self.per_t = None
        self.is_antiper_t = None

    def _get_name_phase(self):
        """getter of name_phase"""
        return self._name_phase

    def _set_name_phase(self, value):
        """setter of name_phase"""
        if type(value) is int and value == -1:
            value = list()
        check_var("name_phase", value, "list")
        self._name_phase = value

    name_phase = property(
        fget=_get_name_phase,
        fset=_set_name_phase,
        doc=u"""Name of the phases of the winding (if any)

        :Type: list
        """,
    )

    def _get_BH_curve(self):
        """getter of BH_curve"""
        return self._BH_curve

    def _set_BH_curve(self, value):
        """setter of BH_curve"""
        if type(value) is int and value == -1:
            value = array([])
        elif type(value) is list:
            try:
                value = array(value)
            except:
                pass
        check_var("BH_curve", value, "ndarray")
        self._BH_curve = value

    BH_curve = property(
        fget=_get_BH_curve,
        fset=_set_BH_curve,
        doc=u"""B(H) curve (two columns matrix, H and B(H))

        :Type: ndarray
        """,
    )

    def _get_Ksfill(self):
        """getter of Ksfill"""
        return self._Ksfill

    def _set_Ksfill(self, value):
        """setter of Ksfill"""
        check_var("Ksfill", value, "float")
        self._Ksfill = value

    Ksfill = property(
        fget=_get_Ksfill,
        fset=_set_Ksfill,
        doc=u"""Slot fill factor

        :Type: float
        """,
    )

    def _get_S_slot(self):
        """getter of S_slot"""
        return self._S_slot

    def _set_S_slot(self, value):
        """setter of S_slot"""
        check_var("S_slot", value, "float")
        self._S_slot = value

    S_slot = property(
        fget=_get_S_slot,
        fset=_set_S_slot,
        doc=u"""Slot surface

        :Type: float
        """,
    )

    def _get_S_slot_wind(self):
        """getter of S_slot_wind"""
        return self._S_slot_wind

    def _set_S_slot_wind(self, value):
        """setter of S_slot_wind"""
        check_var("S_slot_wind", value, "float")
        self._S_slot_wind = value

    S_slot_wind = property(
        fget=_get_S_slot_wind,
        fset=_set_S_slot_wind,
        doc=u"""Slot winding surface

        :Type: float
        """,
    )

    def _get_S_wind_act(self):
        """getter of S_wind_act"""
        return self._S_wind_act

    def _set_S_wind_act(self, value):
        """setter of S_wind_act"""
        check_var("S_wind_act", value, "float")
        self._S_wind_act = value

    S_wind_act = property(
        fget=_get_S_wind_act,
        fset=_set_S_wind_act,
        doc=u"""Conductor active surface

        :Type: float
        """,
    )

    def _get_per_a(self):
        """getter of per_a"""
        return self._per_a

    def _set_per_a(self, value):
        """setter of per_a"""
        check_var("per_a", value, "int")
        self._per_a = value

    per_a = property(
        fget=_get_per_a,
        fset=_set_per_a,
        doc=u"""Number of spatial periodicities of the lamination

        :Type: int
        """,
    )

    def _get_is_antiper_a(self):
        """getter of is_antiper_a"""
        return self._is_antiper_a

    def _set_is_antiper_a(self, value):
        """setter of is_antiper_a"""
        check_var("is_antiper_a", value, "bool")
        self._is_antiper_a = value

    is_antiper_a = property(
        fget=_get_is_antiper_a,
        fset=_set_is_antiper_a,
        doc=u"""True if an spatial anti-periodicity is possible after the periodicities

        :Type: bool
        """,
    )

    def _get_per_t(self):
        """getter of per_t"""
        return self._per_t

    def _set_per_t(self, value):
        """setter of per_t"""
        check_var("per_t", value, "int")
        self._per_t = value

    per_t = property(
        fget=_get_per_t,
        fset=_set_per_t,
        doc=u"""Number of time periodicities of the lamination

        :Type: int
        """,
    )

    def _get_is_antiper_t(self):
        """getter of is_antiper_t"""
        return self._is_antiper_t

    def _set_is_antiper_t(self, value):
        """setter of is_antiper_t"""
        check_var("is_antiper_t", value, "bool")
        self._is_antiper_t = value

    is_antiper_t = property(
        fget=_get_is_antiper_t,
        fset=_set_is_antiper_t,
        doc=u"""True if an time anti-periodicity is possible after the periodicities

        :Type: bool
        """,
    )



from numpy import arcsin, cos, exp, pi, sin, tan, sqrt


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : SlotW14
        A SlotW14 object

    Returns
    -------
    point_dict: dict
        A dict of the slot point coordinates
    """
    Rbo = self.get_Rbo()
    hssp = pi / self.Zs

    # alpha is the angle to rotate P0 so ||P1,P9|| = W0
    alpha = arcsin(self.W0 / (2 * Rbo))
    Harc = float(Rbo * (1 - cos(alpha)))

    Z1 = Rbo * exp(-1j * alpha)
    if self.is_outwards():
        R1 = Rbo - Harc + self.H0 + self.H1
        Z2 = Z1 + self.H0
    else:
        R1 = Rbo - Harc - self.H0 - self.H1
        Z2 = Z1 - self.H0
    # In the slot ref: Z3=x7+j*y7 with x7 = R1
    # In the tooth ref: Zt7 = xt7 + j*yt7 with yt7 = W3/2
    # Zt7 = Z3 * exp(1j*hsp) = (x7+jy7)*(cos(hsp)+1j*sin(hsp))
    # yt7 = W3/2 = x7*sin(hsp)+y7*cos(hsp)
    Z3 = R1 + 1j * (self.W3 / 2 - R1 * sin(hssp)) / cos(hssp)

    # Z3t is Z3 in tooth ref
    Z3t = Z3 * exp(1j * hssp)
    if self.is_outwards():
        Z4t = Z3t + self.H3
    else:
        Z4t = Z3t - self.H3
    # In the slot ref: Z5=x5+j*y5 with y5 = 0
    # In the tooth ref: Zt5 = xt5 + j*yt5 with xt5 = xt6
    # Zt5 = Z5 * exp(1j*hsp) = x5*cos(hsp)+1j*x5*sin(hsp)
    # x5 = real(Z4t)/cos(hsp)

    Z5 = Z4t.real / cos(hssp)
    Z4 = Z4t * exp(-1j * hssp)

    point_dict = dict()
    # symetry
    point_dict["Z1"] = Z1
    point_dict["Z2"] = Z2
    point_dict["Z3"] = Z3
    point_dict["Z4"] = Z4
    point_dict["Z5"] = Z5
    point_dict["Z6"] = Z4.conjugate()
    point_dict["Z7"] = Z3.conjugate()
    point_dict["Z8"] = Z2.conjugate()
    point_dict["Z9"] = Z1.conjugate()

    return point_dict



# -*- coding: utf-8 -*-
import sys
from os.path import dirname, join, isfile
from sys import argv, exit

from PySide2.QtCore import QTranslator
from PySide2.QtWidgets import QApplication
from PySide2.QtGui import QIcon

try:  # Import if pyleecan is installed with pip
    from .definitions import ROOT_DIR, PACKAGE_NAME, config_dict
    from .GUI.Dialog.DMachineSetup.DMachineSetup import DMachineSetup
    from .GUI.Dialog.DMatLib.DMatLib import DMatLib
    from .GUI.Dialog.DMatLib.MatLib import MatLib
    from .GUI.Dialog.DMatLib.WMatSelect.WMatSelect import WMatSelect
    from .GUI.Tools.SidebarWindow import SidebarWindow
    from .GUI.Tools.MachinePlotWidget import MachinePlotWidget
    from .GUI.Tools.TreeView import TreeView
    from .GUI.Tools.GuiOption.WGuiOption import WGuiOption

except ImportError:  # Import for dev version
    from definitions import PACKAGE_NAME, ROOT_DIR, config_dict

    exec(
        "from "
        + PACKAGE_NAME
        + ".GUI.Dialog.DMachineSetup.DMachineSetup import DMachineSetup"
    )
    exec("from " + PACKAGE_NAME + ".GUI.Dialog.DMatLib.DMatLib import DMatLib")
    exec("from " + PACKAGE_NAME + ".GUI.Dialog.DMatLib.MatLib import MatLib")
    exec(
        "from "
        + PACKAGE_NAME
        + ".GUI.Dialog.DMatLib.WMatSelect.WMatSelect import WMatSelect"
    )
    exec("from " + PACKAGE_NAME + ".GUI.Tools.SidebarWindow import SidebarWindow")
    exec(
        "from " + PACKAGE_NAME + ".GUI.Tools.MachinePlotWidget import MachinePlotWidget"
    )
    exec("from " + PACKAGE_NAME + ".GUI.Tools.TreeView import TreeView")
    exec("from " + PACKAGE_NAME + ".GUI.Tools.GuiOption.WGuiOption import WGuiOption")


EXT_GUI = True


def run_GUI(argv):
    # Default material data path

    # Script to be used to test in dev
    a = QApplication(argv)

    # Set CSS
    # a.setStyleSheet("QLineEdit { background-color: yellow }")

    # Setup the translation
    translationFile = "pyleecan_fr.qm"
    translator = QTranslator()
    translator.load(translationFile, "GUI//i18n")
    a.installTranslator(translator)
    if isfile(config_dict["GUI"]["CSS_PATH"]):
        with open(config_dict["GUI"]["CSS_PATH"], "r") as css_file:
            a.setStyleSheet(css_file.read())

    # Setting the material library
    matlib = MatLib(config_dict["MAIN"]["MATLIB_DIR"])

    # MatLib widget
    mat_widget = DMatLib(matlib, selected=0)

    # Machine Setup Widget
    c = DMachineSetup(
        dmatlib=mat_widget, machine_path=config_dict["MAIN"]["MACHINE_DIR"]
    )

    if EXT_GUI:
        # Setup extended GUI with sub windows
        icon = dirname(__file__) + "/GUI/Resources/images/icon/pyleecan_64.png"
        window = SidebarWindow()
        window.setWindowIcon(QIcon(icon))

        update_step = lambda: c.set_nav(c.nav_step.currentRow())
        window.addSubWindow("Design", c, update_step)
        window.DesignWidget = c

        plt_widget = MachinePlotWidget(window)
        window.addSubWindow("Plot", plt_widget, plt_widget.update)

        mat_widget.installEventFilter(window)
        window.addSubWindow("MatLib", mat_widget, mat_widget.update_list_mat)

        tree = TreeView()
        tree_fcn = lambda: tree.generate(getattr(c, "machine"))
        window.addSubWindow("TreeView", tree, tree_fcn)

        option = WGuiOption(machine_setup=c, matlib=matlib)
        window.addSubWindow("Option", option)
        window.show()

    else:
        # "Normal" GUI
        c.show()

    exit(a.exec_())


if __name__ == "__main__":
    run_GUI(argv)



# -*- coding: utf-8 -*-

from numpy import arcsin, pi

from pyleecan.Methods.Slot.Slot import SlotCheckError
from ....Methods.Slot.HoleM57 import *


def check(self):
    """Check that the HoleM57 object is correct

    Parameters
    ----------
    self : HoleM57
        A HoleM57 object

    Returns
    -------
    None

    Raises
    _______
    """
    # Check that everything is set
    if self.W0 is None:
        raise S57_NoneError("You must set W0 !")
    elif self.W1 is None:
        raise S57_NoneError("You must set W1 !")
    elif self.W2 is None:
        raise S57_NoneError("You must set W2 !")
    elif self.W3 is None:
        raise S57_NoneError("You must set W3 !")
    elif self.W4 is None:
        raise S57_NoneError("You must set W4 !")
    elif self.H1 is None:
        raise S57_NoneError("You must set H1 !")
    elif self.H2 is None:
        raise S57_NoneError("You must set H2 !")



# -*- coding: utf-8 -*-
from numpy import abs as np_abs, exp


def get_end(self):
    """Return the end of the arc

    Parameters
    ----------
    self : Arc2
        An Arc2 object

    Returns
    -------
    end: complex
        Complex coordinates of the end of the Arc2
    """

    self.check()

    # the center is on the bisection of [begin, end]
    z1 = self.begin
    zc = self.center
    angle = self.angle

    # Geometric transformation : center is the origine (-zc)
    # Then rotation of begin of the correct angle (*exp(i*pi*angle))
    # Then return to the main axis (+zc)
    z2 = (z1 - zc) * exp(1j * angle) + zc

    # Return (0,0) if the point is too close from 0
    if np_abs(z2) < 1e-6:
        z2 = 0
    return z2



# -*- coding: utf-8 -*-

from os import getcwd, remove, rename
from os.path import abspath, dirname, join, split
from re import match

from PySide2.QtCore import Signal
from PySide2.QtWidgets import QDialog, QFileDialog, QListWidget, QMessageBox

from ....Classes.ImportMatrixVal import ImportMatrixVal
from ....Classes.ImportMatrixXls import ImportMatrixXls
from ....Functions.load import load_matlib
from ....Functions.path_tools import abs_file_path
from ....GUI.Dialog.DMatLib.DMatSetup.DMatSetup import DMatSetup
from ....GUI.Dialog.DMatLib.Gen_DMatLib import Gen_DMatLib


class DMatLib(Gen_DMatLib, QDialog):
    """Material Library Dialog to view and modify material data."""

    # Signal to W_MachineSetup to know that the save popup is needed
    saveNeeded = Signal()

    def __init__(self, matlib, key="RefMatLib", selected=0):
        """Init the Matlib GUI

        Parameters
        ----------
        self :
            a DMatLib object
        matlib : MatLib
            The current state of the Material library (list of
            material)
        selected :
            Index of the currently selected material

        Returns
        -------

        """

        # Build the interface according to the .ui file
        QDialog.__init__(self)
        self.setupUi(self)

        # Load the material library
        self.matlib = matlib
        self.current_dialog = None

        # bool to know if the material selected is in MatLib
        self.mat_selected_in_matlib = True

        # Unknow material => Use the first one
        if selected == -1:
            selected = 0
        self.update_list_mat()

        # Select the material
        if key == "RefMatLib":
            self.nav_mat.setCurrentRow(selected)
            self.update_out(on_matlib=True)
        else:
            self.nav_mat_mach.setCurrentRow(selected)
            self.update_out(on_matlib=False)

        # Hide since unused
        # self.out_iso.hide()
        self.out_epsr.hide()

        self.nav_mat.itemSelectionChanged.connect(
            lambda: self.update_out(on_matlib=True)
        )
        self.nav_mat_mach.itemSelectionChanged.connect(
            lambda: self.update_out(on_matlib=False)
        )

        self.le_search.textChanged.connect(self.filter_material)

        self.b_edit.clicked.connect(self.edit_material)
        self.b_delete.clicked.connect(self.delete_material)
        self.b_duplicate.clicked.connect(self.new_material)
        self.nav_mat.doubleClicked.connect(self.edit_material)
        self.nav_mat_mach.doubleClicked.connect(self.edit_material)

    def load(self):
        """Load the material library

        Parameters
        ----------
        self :
            A DMatLib object

        Returns
        -------

        """

        load_path = str(
            QFileDialog.getExistingDirectory(
                self, self.tr("Select Material Library Directory"), self.matlib_path
            )
        )

        if load_path != "":
            try:
                self.matlib.load_mat_ref(load_path)
                self.update_list_mat()
                self.nav_mat.setCurrentRow(0)
                self.update_out()
                self.matlib_path = dirname(load_path)
            except Exception as e:
                QMessageBox().critical(
                    self,
                    self.tr("Error"),
                    self.tr(
                        "The material library file is " "incorrect:\n",
                        "Please keep the \n, another " "message is following this one",
                    )
                    + type(e).__name__
                    + ": "
                    + str(e),
                )
                self.reject()
        else:
            self.reject()

    def edit_material(self):
        """
        Open the setup material GUI to edit the current material.
        Changes will be saved to the corresponding material file.

        Parameters
        ----------
        self :
            A DMatLib object
        """

        # Get the current material
        if self.nav_mat.currentRow() != -1:
            mat_id = self.nav_mat.currentRow()
            dict_key = "RefMatLib"
        elif self.nav_mat_mach.currentRow() != -1:
            mat_id = self.nav_mat_mach.currentRow()
            dict_key = "MachineMatLib"
        else:
            mat_id = 0
            dict_key = "RefMatLib"

        # Close previous window if needed
        if self.current_dialog is not None:
            self.current_dialog.close()

        # (creates a copy of the material, i.e. self.matlib won't be edited directly)
        self.current_dialog = DMatSetup(
            self.matlib.dict_mat[dict_key][mat_id], self.mat_selected_in_matlib
        )
        self.current_dialog.finished.connect(self.validate_edit)
        self.current_dialog.dict_key = dict_key
        self.current_dialog.mat_id = mat_id
        self.current_dialog.show()

    def validate_edit(self, return_code):
        dict_key = self.current_dialog.dict_key
        mat_id = self.current_dialog.mat_id
        mat = self.current_dialog.mat
        # Reset dialog
        self.current_dialog = None
        if return_code > 0:
            # Change the material
            is_change = self.matlib.replace_material(
                dict_key, mat_id, mat, save=return_code == 1
            )

            # Emit saveNeeded if the machine has been change
            if is_change:
                self.saveNeeded.emit()

            # Move the material
            if return_code == 2:
                if dict_key == "MachineMatLib":
                    # Move the material into the Material library
                    self.matlib.move_mach_mat_to_ref(dict_key, mat_id)

                    # Set the pointer on the ref matlib if machine material list is empty
                    if len(self.matlib.dict_mat["MachineMatLib"]) == 0:
                        self.on_matlib = True
                else:
                    # Move the material into the machine materials list
                    self.matlib.move_ref_mat_to_mach("MachineMatLib", mat_id)

            # Update material list
            self.update_list_mat()

            if self.mat_selected_in_matlib:
                self.nav_mat.setCurrentRow(self.nav_mat.count() - 1)
                self.update_out(on_matlib=True)
            else:
                self.nav_mat_mach.setCurrentRow(self.nav_mat_mach.count() - 1)
                self.update_out(on_matlib=False)

            # Signal set by WMatSelect to update Combobox
            self.accepted.emit()

    def new_material(self):
        """Open the setup material GUI to create a new material according to
        the current material

        Parameters
        ----------
        self :
            A DMatLib object

        Returns
        -------

        """
        # Get the current material (filtering index)
        if self.mat_selected_in_matlib:
            mat_id = int(self.nav_mat.currentItem().text()[:3]) - 1
            dict_key = "RefMatLib"
        else:
            mat_id = self.nav_mat_mach.currentRow()
            dict_key = "MachineMatLib"

        # Close previous window if needed
        if self.current_dialog is not None:
            self.current_dialog.close()

        # (creates a copy of the material, i.e. self.matlib won't be edited directly)
        self.current_dialog = DMatSetup(
            self.matlib.dict_mat[dict_key][mat_id], self.mat_selected_in_matlib
        )
        self.current_dialog.finished.connect(self.validate_new)
        self.current_dialog.show()

    def validate_new(self, return_code):
        mat = self.current_dialog.mat
        # Reset dialog
        self.current_dialog = None
        if return_code > 0:
            # Update the material only if the user validate at the end
            if (self.mat_selected_in_matlib and return_code == 1) or (
                not self.mat_selected_in_matlib and return_code == 2
            ):
                self.mat_selected_in_matlib = True

                # Add the material to the mat_ref
                self.matlib.add_new_mat_ref(mat)

                self.update_list_mat()
                self.nav_mat.setCurrentRow(self.nav_mat.count() - 1)
                self.update_out()

            else:
                # Add the material to the machine materials
                self.matlib.add_new_mat_mach(mat)

                self.update_list_mat()
                self.nav_mat_mach.setCurrentRow(self.nav_mat_mach.count() - 1)
                self.update_out(on_matlib=False)

            # Signal set by WMatSelect to update Combobox
            self.accepted.emit()

    def delete_material(self):
        """Delete the selected material

        Parameters
        ----------
        self :
            A DMatLib object

        Returns
        -------

        """
        if self.nav_mat.currentRow() != -1:
            dict_key = "RefMatLib"
            mat_id = self.nav_mat.currentRow()
        else:
            dict_key = "MachineMatLib"
            mat_id = self.nav_mat_mach.currentRow()
            self.mat_selected_in_matlib = False
        # Do not delete machine material
        # if not self.mat_selected_in_matlib:
        #     QMessageBox.information(self, "", "Cannot delete machine material.")
        #     return

        if len(self.matlib.dict_mat["RefMatLib"]) > 1 or dict_key != "RefMatLib":

            del_msg = (
                "Are you sure you want to delete "
                + self.matlib.dict_mat[dict_key][mat_id].name
                + " ?"
            )
            reply = QMessageBox.question(
                self, "Confirmation", del_msg, QMessageBox.Yes, QMessageBox.No
            )

            if reply == QMessageBox.Yes:
                # delete only if confirmed
                self.matlib.delete_material(dict_key, mat_id)
                index = self.nav_mat.currentRow()
                self.update_list_mat()
                if index < self.nav_mat.count() - 1:
                    self.nav_mat.setCurrentRow(index)
                else:
                    self.nav_mat.setCurrentRow(self.nav_mat.count() - 1)
                # Signal set by WMatSelect to update Combobox
                self.accepted.emit()
                self.update_list_mat()
        else:
            QMessageBox.information(
                self, "", "Reference MatLib must contain at least one material."
            )

    def filter_material(self, index=None):
        """

        Parameters
        ----------
        index :
             (Default value = None)

        Returns
        -------

        """
        self.update_list_mat()
        self.nav_mat.setCurrentRow(0)

    def update_list_mat(self, selected_id=-1):
        """Update the list of Material with the current content of MatLib

        Parameters
        ----------
        self :
            A DMatLib object

        Returns
        -------

        """
        dict_mat = self.matlib.dict_mat
        self.nav_mat.blockSignals(True)
        self.nav_mat.clear()

        # Filter the material
        for ii, mat in enumerate(dict_mat["RefMatLib"]):
            # todo: add new filter
            if self.le_search.text() != "" and not match(
                ".*" + self.le_search.text().lower() + ".*", mat.name.lower()
            ):
                continue
            self.nav_mat.addItem("%03d" % (ii + 1) + " - " + mat.name)
        self.nav_mat.blockSignals(False)

        self.nav_mat_mach.blockSignals(True)
        self.nav_mat_mach.clear()

        # Filter the material
        for ii, mat in enumerate(dict_mat["MachineMatLib"]):
            # todo: add new filter
            if self.le_search.text() != "" and not match(
                ".*" + self.le_search.text().lower() + ".*", mat.name.lower()
            ):
                continue
            self.nav_mat_mach.addItem(
                "%03d" % (len(dict_mat["RefMatLib"]) + ii + 1) + " - " + mat.name
            )
        self.nav_mat_mach.blockSignals(False)
        # Hide the widget if machine material list is empty
        if len(dict_mat["MachineMatLib"]) == 0:
            self.in_machine_mat.setVisible(False)
            self.nav_mat_mach.setVisible(False)
        elif not self.in_machine_mat.isVisible():
            self.in_machine_mat.setVisible(True)
            self.nav_mat_mach.setVisible(True)

    def update_out(self, on_matlib=True):
        """Update all the output widget for material preview

        Parameters
        ----------
        self :
            A DMatLib object
        index :
            Current index of nav_mat (Default value = 0)

        Returns
        -------

        """
        # Get the selected material
        if on_matlib:
            # Return if no material is selected
            if self.nav_mat.currentRow() == -1 and self.nav_mat_mach.currentRow() != -1:
                return
            elif self.nav_mat.currentRow() == -1:
                self.nav_mat.setCurrentRow(0)

            # Set dict_key and mat_id to select the right material in MatLib
            dict_key = "RefMatLib"
            mat_id = self.nav_mat.currentRow()

            self.mat_selected_in_matlib = True

            # Deselect item in nav_mat_mach by setting its current row to -1
            self.nav_mat_mach.setCurrentRow(-1)
        else:
            # Return if no material is selected
            if self.nav_mat_mach.currentRow() == -1:
                return

            # Set dict_key and mat_id to select the right material in MatLib
            dict_key = "MachineMatLib"
            mat_id = self.nav_mat_mach.currentRow()

            self.mat_selected_in_matlib = False

            # Deselect item in nav_mat by setting its current row to -1
            self.nav_mat.setCurrentRow(-1)

        mat = self.matlib.dict_mat[dict_key][mat_id]

        # Update Main parameters
        self.out_name.setText(self.tr("name: ") + mat.name)
        if mat.is_isotropic:
            self.out_iso.setText(self.tr("type: isotropic"))
        else:
            self.out_iso.setText(self.tr("type: anisotropic"))

        # Update Electrical parameters
        if mat.elec is not None:
            update_text(self.out_rho_elec, "rho", mat.elec.rho, "ohm.m")
            # update_text(self.out_epsr,"epsr",mat.elec.epsr,None)

        # Update Economical parameters
        if mat.eco is not None:
            update_text(self.out_cost_unit, "cost_unit", mat.eco.cost_unit, u"€/kg")

        # Update Thermics parameters
        if mat.HT is not None:
            update_text(self.out_Cp, "Cp", mat.HT.Cp, "W/kg/K")
            update_text(self.out_alpha, "alpha", mat.HT.alpha, None)
            if mat.is_isotropic:
                self.nav_iso_therm.setCurrentIndex(0)
                update_text(self.out_L, "Lambda", mat.HT.lambda_x, "W/K")
            else:
                self.nav_iso_therm.setCurrentIndex(1)
                update_text(self.out_LX, "Lambda X", mat.HT.lambda_x, "W/K")
                update_text(self.out_LY, "Lambda Y", mat.HT.lambda_y, "W/K")
                update_text(self.out_LZ, "Lambda Z", mat.HT.lambda_z, "W/K")

        # Update Structural parameters
        if mat.struct is not None:
            update_text(self.out_rho_meca, "rho", mat.struct.rho, "kg/m^3")
            if mat.is_isotropic:
                self.nav_iso_meca.setCurrentIndex(0)
                update_text(self.out_E, "E", mat.struct.Ex, "Pa")
                update_text(self.out_G, "G", mat.struct.Gxy, "Pa")
                update_text(self.out_nu, "nu", mat.struct.nu_xy, None)
            else:
                self.nav_iso_meca.setCurrentIndex(1)
                update_text(self.out_EX, None, mat.struct.Ex, None)
                update_text(self.out_GXY, None, mat.struct.Gxy, None)
                update_text(self.out_nu_XY, None, mat.struct.nu_xy, None)
                update_text(self.out_EY, None, mat.struct.Ey, None)
                update_text(self.out_GYZ, None, mat.struct.Gyz, None)
                update_text(self.out_nu_YZ, None, mat.struct.nu_yz, None)
                update_text(self.out_EZ, None, mat.struct.Ez, None)
                update_text(self.out_GXZ, None, mat.struct.Gxz, None)
                update_text(self.out_nu_XZ, None, mat.struct.nu_xz, None)

        # Update Magnetics parameters
        if mat.mag is not None:
            update_text(self.out_mur_lin, "mur_lin", mat.mag.mur_lin, None)
            update_text(self.out_Brm20, "Brm20", mat.mag.Brm20, "T")
            update_text(self.out_alpha_Br, "alpha_Br", mat.mag.alpha_Br, None)
            update_text(self.out_wlam, "wlam", mat.mag.Wlam, "m")
            if isinstance(mat.mag.BH_curve, ImportMatrixXls):
                BH_text = split(mat.mag.BH_curve.file_path)[1]
            elif isinstance(mat.mag.BH_curve, ImportMatrixVal):
                data = mat.mag.BH_curve.get_data()
                shape_str = str(data.shape) if data is not None else "(-,-)"
                BH_text = "Matrix " + shape_str
            else:
                BH_text = "-"
            self.out_BH.setText(BH_text)

    def closeEvent(self, event):
        """Display a message before leaving

        Parameters
        ----------
        self : DMatSetup
            A DMatSetup object
        event :
            The closing event
        """
        # Close popup if needed
        if self.current_dialog is not None:
            self.current_dialog.close()


def update_text(label, name, value, unit):
    """Update a Qlabel with the value if not None

    Parameters
    ----------
    label :
        Qlabel to update
    name :
        Name of the variable
    value :
        Current value of the variable (can be None)
    unit :
        Variable unit (can be None)

    Returns
    -------

    """
    if value is None:
        val = "?"
    else:
        val = str(value)
        if len(val) > 8:
            val = "%1.2e" % value  # formating: 1.23e+08

    if name is None:  # For E, G, nu table
        txt = val
    else:
        if unit is None:
            txt = name + " = " + val
        else:
            txt = name + " = " + val + " " + unit
    label.setText(txt)



# -*- coding: utf-8 -*-
from ....Methods.Slot.Slot import SlotCheckError


class S28_WCheckError(SlotCheckError):
    """ """

    pass


class S28_RboW0CheckError(SlotCheckError):
    """ """

    pass


class S28_R1W3CheckError(SlotCheckError):
    """ """

    pass


class S28_R1R1CheckError(SlotCheckError):
    """ """

    pass


class S28_ZsCheckError(SlotCheckError):
    """ """

    pass



# -*- coding: utf-8 -*-

import numpy as np


def get_ref_point(self, vertice, point):
    """Return the coordinate of the equivalent point in the ref cell

    Parameters
    ----------
    self : RefTriangle3
        a RefTriangle3 object
    vertice : ndarray
        vertice of the cell
    point : ndarray
        coordinates of a point

        Returns
    -------
    pt1_ref : ndarray
        coordinates of the ref point
    """

    [jacob, detJ] = self.jacobian(point, vertice)
    inv_jacob = np.linalg.inv(jacob)
    point_ref = np.dot((point - vertice[0, :]), inv_jacob)

    return point_ref



# -*- coding: utf-8 -*-


def as_dict(self):
    """Convert this object in a json seriable dict (can be use in __init__)"""

    # Get the properties inherited from Mesh
    MeshVTK_dict = super(type(self), self).as_dict()
    MeshVTK_dict["mesh"] = None
    MeshVTK_dict["is_pyvista_mesh"] = self.is_pyvista_mesh
    MeshVTK_dict["format"] = self.format
    MeshVTK_dict["path"] = self.path
    MeshVTK_dict["name"] = self.name
    MeshVTK_dict["surf"] = None
    MeshVTK_dict["is_vtk_surf"] = self.is_vtk_surf
    MeshVTK_dict["surf_path"] = self.surf_path
    MeshVTK_dict["surf_name"] = self.surf_name
    # The class name is added to the dict for deserialisation purpose
    # Overwrite the mother class name
    MeshVTK_dict["__class__"] = "MeshVTK"
    return MeshVTK_dict



# -*- coding: utf-8 -*-
from SciDataTool import DataTime, VectorField


def store(self, out_dict, axes_dict):
    """Store the standard outputs of Force that are temporarily in out_dict as arrays into OutForce as Data object

    Parameters
    ----------
    self: OutForce
        the OutForce object to update
    out_dict : dict
        Dict containing all force quantities
    axes_dict: {Data}
        Dict of axes used for magnetic calculation

    """

    # Store air-gap surface force as VectorField object

    # Axes for each component
    axis_list = [axes_dict["Time"], axes_dict["Angle"]]

    # Create VectorField with empty components
    self.AGSF = VectorField(
        name="Air gap Surface Force",
        symbol="AGSF",
    )
    # Radial air-gap surface force component
    if "AGSF_r" in out_dict:
        self.AGSF.components["radial"] = DataTime(
            name="Radial AGSF",
            unit="N/m²",
            symbol="AGSF_r",
            axes=axis_list,
            values=out_dict.pop("AGSF_r"),
        )
    # Tangential air-gap surface force component
    if "AGSF_t" in out_dict:
        self.AGSF.components["tangential"] = DataTime(
            name="Tangential AGSF",
            unit="N/m²",
            symbol="AGSF_t",
            axes=axis_list,
            values=out_dict.pop("AGSF_t"),
        )
    # Axial air-gap surface force component
    if "AGSF_z" in out_dict:
        self.AGSF.components["axial"] = DataTime(
            name="Axial AGSF",
            unit="N/m²",
            symbol="AGSF_z",
            axes=axis_list,
            values=out_dict.pop("AGSF_z"),
        )

    if "Rag" in out_dict:
        self.Rag = out_dict.pop("Rag")



# -*- coding: utf-8 -*-
from ....Functions.Geometry.comp_surface_num import comp_surface_num


def comp_surface(self, Ndisc=200):
    """Compute the SurfLine surface

    Parameters
    ----------
    self : SurfLine
        A SurfLine object
    Ndisc : int
        Number of point to discretize the lines

    Returns
    -------
    surf: float
        The SurfLine surface [m**2]

    """

    # Discretize the surface with lots of points to compute the surface numerically
    point_list = self.discretize(Ndisc)

    return comp_surface_num(point_list)



# -*- coding: utf-8 -*-

from numpy import exp


def comp_radius(self):
    """Compute the radius of the min and max circle that contains the slot

    Parameters
    ----------
    self : Slot
        A Slot object

    Returns
    -------
    (Rmin,Rmax): tuple
        Radius of the circle that contains the slot [m]

    """

    Rbo = self.get_Rbo()

    H = self.comp_height()
    Hmag = self.comp_height_active()

    if H < Hmag:
        if self.is_outwards():
            return (Rbo - Hmag + H, Rbo + H)
        else:
            return (Rbo - H, Rbo + Hmag - H)
    else:
        if self.is_outwards():
            return (Rbo, Rbo + H)
        else:
            return (Rbo - H, Rbo)



# -*- coding: utf-8 -*-
from ....Classes.Segment import Segment
from ....Classes.Arc1 import Arc1


def build_geometry(self):
    """Compute the curve (Line) needed to plot the Slot.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : SlotM14
        A SlotM14 object

    Returns
    -------
    curve_list: list
        A list of 3 Segments

    """
    Rbo = self.get_Rbo()
    point_dict = self._comp_point_coordinate()
    Z1 = point_dict["Z1"]
    Z2 = point_dict["Z2"]
    Z3 = point_dict["Z3"]
    Z4 = point_dict["Z4"]

    # Creation of curve
    curve_list = list()
    if self.H0 > 0:
        curve_list.append(Segment(Z1, Z2))
    if self.is_outwards():
        curve_list.append(Arc1(Z2, Z3, Rbo + self.H0, is_trigo_direction=True))
    else:
        curve_list.append(Arc1(Z2, Z3, Rbo - self.H0, is_trigo_direction=True))
    if self.H0 > 0:
        curve_list.append(Segment(Z3, Z4))

    return curve_list



# -*- coding: utf-8 -*-

from ....Classes.Arc1 import Arc1
from ....Classes.Arc2 import Arc2
from ....Classes.Arc3 import Arc3
from ....Functions.FEMM.get_mesh_param import get_mesh_param


def draw_FEMM(
    self, femm, nodeprop=None, maxseg=None, propname=None, FEMM_dict=None, hide=False
):
    """draw the Surface in FEMM

    Parameters
    ----------
    femm : FEMMHandler
        client to send command to a FEMM instance
    nodeprop :
        Nodal property
         (Default value = None)
    maxseg :
        Meshed with elements that span at most maxsegdeg degrees per element
         (Default value = None)
    propname :
        Boundary property ’propname’
         (Default value = None)
    FEMM_dict : dict
        Dictionnary containing the main parameters of FEMM
    hide :
        0 = not hidden in post-processor, 1 == hidden in post processor
        (Default value = False)

    Returns
    -------
    None

    """
    # Check if the Surface is correct
    self.check()

    # Draw all the lines
    lines = self.get_lines()

    for line in lines:
        if line.label in [None, ""]:  # Use surface mesh param
            mesh_dict = get_mesh_param(self.label, FEMM_dict)
        else:  # Use line mesh param
            mesh_dict = get_mesh_param(line.label, FEMM_dict)
        if type(line) in [Arc1, Arc2, Arc3]:
            line.draw_FEMM(
                femm=femm,
                nodeprop=nodeprop,
                maxseg=maxseg,
                propname=propname,
                hide=hide,
                group=mesh_dict["group"],
            )
        else:
            line.draw_FEMM(
                femm=femm,
                nodeprop=nodeprop,
                propname=propname,
                element_size=mesh_dict["element_size"],
                automesh=mesh_dict["automesh"],
                hide=hide,
                group=mesh_dict["group"],
            )



from numpy import pi
import matplotlib.pyplot as plt
import matplotlib.animation as animation
from matplotlib.pyplot import axis, subplots
from matplotlib.patches import Polygon, Circle


def plot_anim_rotor(self, Nframe, Tanim, Nrot=1, is_loop=True):
    """Plot the machine with an animation of the rotor
    (inner rotor for now ?)

    Parameters
    ----------
    self : Machine
        Machine object
    Nframe: int
        Number of frame for the animation
    Tanim : float
        Duration of the animation [ms]
    Nrot : float
        Number of rotation
    is_loop : bool
        True to activate the loop animation
    """

    # Display
    fig, axes = subplots()
    axes.set_xlabel("(m)")
    axes.set_ylabel("(m)")
    axes.set_title("Machine")

    # Axis Setup
    axis("equal")

    # The Lamination is centered in the figure
    Lim = (self.stator.Rext) * 1.5  # Axes limit for plot
    Rsurf = self.rotor.build_geometry(sym=1, alpha=0, delta=0)

    # Rotation angle between each frame
    Dalpha = 2 * pi * Nrot / Nframe

    def init():
        """Create the patches for the first image"""
        Spatches = self.stator.plot(is_display=False)
        Rpatches = self.rotor.plot(is_display=False)
        for patch in Spatches:
            axes.add_patch(patch)
        for patch in Rpatches:
            axes.add_patch(patch)
        return []

    def update_rotor(ii):
        """Rotate and update the rotor patches"""
        for ii in range(len(Rsurf)):
            Rsurf[ii].rotate(Dalpha)
            patches = Rsurf[ii].get_patches()
            for patch in patches:
                if type(patch) is Polygon:
                    axes.patches[-len(Rsurf) + ii].xy = patch.xy
            # elif type(patch) is Circle:
            #     pass

        axes.set_xlim(-Lim, Lim)
        axes.set_ylim(-Lim, Lim)
        return []

    # Animation definition
    anim = animation.FuncAnimation(
        fig,
        update_rotor,
        init_func=init,
        frames=Nframe,
        interval=Tanim / Nframe,
        blit=True,
        repeat=is_loop,
    )
    plt.show()



# -*- coding: utf-8 -*-

from numpy import pi

from ....Methods.Slot.Slot import SlotCheckError
from ....Methods.Slot.Slot19 import *


def check(self):
    """Check that the Slot19 object is correct

    Parameters
    ----------
    self : Slot19
        A Slot19 object

    Returns
    -------
    None

    Raises
    -------
    S19_H2CheckError
        H1 must be > 0

    S19_W1CheckError
        W1 must be > 0
    """
    if self.W1 < 0:
        raise S19_W1CheckError("W1 must be larger than zero")



# -*- coding: utf-8 -*-
from numpy import take
from SciDataTool import Data1D


def get_solution(self, indice=None):
    """Return a copy of the solution with the option to only include specified indices.

    Parameters
    ----------
    self : SolutionVector
        a SolutionVector object
    indices : list
        list of indices, if list is empty or None all indices are included

    Returns
    -------
    solution: SolutionMat
        solution

    """
    logger = self.get_logger()

    # create copy to directly manipulate data
    solution = self.copy()

    if indice:
        for field in solution.field.components.values():
            # get axes data
            axes = field.axes
            axes_names = [axis.name for axis in field.axes]
            ax_idx = axes_names.index("indice")

            org_indice = axes[ax_idx].get_values()

            if set(indice) - set(org_indice):
                logger.warning(
                    "At least one input indice is not part of the solution. "
                    + "Respective indice will be skipped."
                )

            # skip indice that are not part of the solution
            new_indice = [ii for ii in indice if ii in org_indice]

            # create requested axes list to get field values (see SciDataTool ref.)
            args = [name for name in axes_names]
            args[ax_idx] += "=axis_data"

            # get the field values
            field_dict = field.get_along(*args, axis_data={"indice": new_indice})

            # set new indice axis
            axes[ax_idx] = Data1D(
                values=new_indice,
                is_components=axes[ax_idx].is_components,
                symmetries=axes[ax_idx].symmetries,
                symbol=axes[ax_idx].symbol,
                name=axes[ax_idx].name,
                unit=axes[ax_idx].unit,
                normalizations=axes[ax_idx].normalizations,
            )

            # set new field data
            field.values = field_dict[field.symbol]

    return solution



# -*-- coding: utf-8 -*
def discretize(self, Npoint=-1):
    """Returns the discretize version of the SurfLine

    Parameters
    ----------
    self: SurfLine
        A SurfLine object
    Npoint : int
        Number of point on each line (Default value = -1 => use the line default discretization)

    Returns
    -------
    point_list : list
        List of complex coordinates
    """
    # check if the SurfLine is correct
    self.check()
    # getting lines that delimit the SurfLine
    lines = self.get_lines()
    if lines[0].get_begin() == lines[-1].get_end():
        closed = True
    else:
        closed = False

    point_list = list()
    for line in lines:
        if Npoint == -1:
            point_list.extend(line.discretize())
        else:
            point_list.extend(line.discretize(Npoint))
        if line != lines[-1]:
            point_list.pop()
        else:
            if closed:  # if the SurfLine is closed
                point_list.pop()
    return point_list



from ....Classes.Arc1 import Arc1
from ....Classes.SurfLine import SurfLine


def get_surface_active(self, alpha=0, delta=0):
    """Return the full winding surface

    Parameters
    ----------
    self : SlotW24
        A SlotW24 object
    alpha : float
        float number for rotation (Default value = 0) [rad]
    delta : complex
        complex number for translation (Default value = 0)

    Returns
    -------
    surf_wind: Surface
        Surface corresponding to the Winding Area
    """

    # get the name of the lamination
    st = self.get_name_lam()

    # Create curve list
    curve_list = self.build_geometry()
    curve_list.append(
        Arc1(
            begin=curve_list[-1].get_end(),
            end=curve_list[0].get_begin(),
            radius=-abs(curve_list[-1].get_end()),
            is_trigo_direction=False,
        )
    )

    # Create surface
    if self.is_outwards():
        Zmid = self.get_Rbo() + self.H2 / 2
    else:
        Zmid = self.get_Rbo() - self.H2 / 2
    surface = SurfLine(
        line_list=curve_list, label="Wind_" + st + "_R0_T0_S0", point_ref=Zmid
    )

    # Apply transformation
    surface.rotate(alpha)
    surface.translate(delta)

    return surface



# -*- coding: utf-8 -*-
def check(self):
    """assert the Surface is correct

    Parameters
    ----------
    self : SurfLine
        A SurfLine object

    Returns
    -------
    None
    """
    if self.line_list != []:
        for line in self.line_list:
            line.check()



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Material/Material.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Material/Material
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

from ._check import InitUnKnowClassError
from .MatElectrical import MatElectrical
from .MatMagnetics import MatMagnetics
from .MatStructural import MatStructural
from .MatHT import MatHT
from .MatEconomical import MatEconomical


class Material(FrozenClass):

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        name="Material",
        is_isotropic=False,
        elec=-1,
        mag=-1,
        struct=-1,
        HT=-1,
        eco=-1,
        desc="Material description",
        path="",
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "is_isotropic" in list(init_dict.keys()):
                is_isotropic = init_dict["is_isotropic"]
            if "elec" in list(init_dict.keys()):
                elec = init_dict["elec"]
            if "mag" in list(init_dict.keys()):
                mag = init_dict["mag"]
            if "struct" in list(init_dict.keys()):
                struct = init_dict["struct"]
            if "HT" in list(init_dict.keys()):
                HT = init_dict["HT"]
            if "eco" in list(init_dict.keys()):
                eco = init_dict["eco"]
            if "desc" in list(init_dict.keys()):
                desc = init_dict["desc"]
            if "path" in list(init_dict.keys()):
                path = init_dict["path"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.name = name
        self.is_isotropic = is_isotropic
        self.elec = elec
        self.mag = mag
        self.struct = struct
        self.HT = HT
        self.eco = eco
        self.desc = desc
        self.path = path

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Material_str = ""
        if self.parent is None:
            Material_str += "parent = None " + linesep
        else:
            Material_str += "parent = " + str(type(self.parent)) + " object" + linesep
        Material_str += 'name = "' + str(self.name) + '"' + linesep
        Material_str += "is_isotropic = " + str(self.is_isotropic) + linesep
        if self.elec is not None:
            tmp = self.elec.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Material_str += "elec = " + tmp
        else:
            Material_str += "elec = None" + linesep + linesep
        if self.mag is not None:
            tmp = self.mag.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Material_str += "mag = " + tmp
        else:
            Material_str += "mag = None" + linesep + linesep
        if self.struct is not None:
            tmp = self.struct.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Material_str += "struct = " + tmp
        else:
            Material_str += "struct = None" + linesep + linesep
        if self.HT is not None:
            tmp = self.HT.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Material_str += "HT = " + tmp
        else:
            Material_str += "HT = None" + linesep + linesep
        if self.eco is not None:
            tmp = self.eco.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Material_str += "eco = " + tmp
        else:
            Material_str += "eco = None" + linesep + linesep
        Material_str += 'desc = "' + str(self.desc) + '"' + linesep
        Material_str += 'path = "' + str(self.path) + '"' + linesep
        return Material_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.name != self.name:
            return False
        if other.is_isotropic != self.is_isotropic:
            return False
        if other.elec != self.elec:
            return False
        if other.mag != self.mag:
            return False
        if other.struct != self.struct:
            return False
        if other.HT != self.HT:
            return False
        if other.eco != self.eco:
            return False
        if other.desc != self.desc:
            return False
        if other.path != self.path:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if other._name != self._name:
            diff_list.append(name + ".name")
        if other._is_isotropic != self._is_isotropic:
            diff_list.append(name + ".is_isotropic")
        if (other.elec is None and self.elec is not None) or (
            other.elec is not None and self.elec is None
        ):
            diff_list.append(name + ".elec None mismatch")
        elif self.elec is not None:
            diff_list.extend(self.elec.compare(other.elec, name=name + ".elec"))
        if (other.mag is None and self.mag is not None) or (
            other.mag is not None and self.mag is None
        ):
            diff_list.append(name + ".mag None mismatch")
        elif self.mag is not None:
            diff_list.extend(self.mag.compare(other.mag, name=name + ".mag"))
        if (other.struct is None and self.struct is not None) or (
            other.struct is not None and self.struct is None
        ):
            diff_list.append(name + ".struct None mismatch")
        elif self.struct is not None:
            diff_list.extend(self.struct.compare(other.struct, name=name + ".struct"))
        if (other.HT is None and self.HT is not None) or (
            other.HT is not None and self.HT is None
        ):
            diff_list.append(name + ".HT None mismatch")
        elif self.HT is not None:
            diff_list.extend(self.HT.compare(other.HT, name=name + ".HT"))
        if (other.eco is None and self.eco is not None) or (
            other.eco is not None and self.eco is None
        ):
            diff_list.append(name + ".eco None mismatch")
        elif self.eco is not None:
            diff_list.extend(self.eco.compare(other.eco, name=name + ".eco"))
        if other._desc != self._desc:
            diff_list.append(name + ".desc")
        if other._path != self._path:
            diff_list.append(name + ".path")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.name)
        S += getsizeof(self.is_isotropic)
        S += getsizeof(self.elec)
        S += getsizeof(self.mag)
        S += getsizeof(self.struct)
        S += getsizeof(self.HT)
        S += getsizeof(self.eco)
        S += getsizeof(self.desc)
        S += getsizeof(self.path)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        Material_dict = dict()
        Material_dict["name"] = self.name
        Material_dict["is_isotropic"] = self.is_isotropic
        if self.elec is None:
            Material_dict["elec"] = None
        else:
            Material_dict["elec"] = self.elec.as_dict(**kwargs)
        if self.mag is None:
            Material_dict["mag"] = None
        else:
            Material_dict["mag"] = self.mag.as_dict(**kwargs)
        if self.struct is None:
            Material_dict["struct"] = None
        else:
            Material_dict["struct"] = self.struct.as_dict(**kwargs)
        if self.HT is None:
            Material_dict["HT"] = None
        else:
            Material_dict["HT"] = self.HT.as_dict(**kwargs)
        if self.eco is None:
            Material_dict["eco"] = None
        else:
            Material_dict["eco"] = self.eco.as_dict(**kwargs)
        Material_dict["desc"] = self.desc
        Material_dict["path"] = self.path
        # The class name is added to the dict for deserialisation purpose
        Material_dict["__class__"] = "Material"
        return Material_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.name = None
        self.is_isotropic = None
        if self.elec is not None:
            self.elec._set_None()
        if self.mag is not None:
            self.mag._set_None()
        if self.struct is not None:
            self.struct._set_None()
        if self.HT is not None:
            self.HT._set_None()
        if self.eco is not None:
            self.eco._set_None()
        self.desc = None
        self.path = None

    def _get_name(self):
        """getter of name"""
        return self._name

    def _set_name(self, value):
        """setter of name"""
        check_var("name", value, "str")
        self._name = value

    name = property(
        fget=_get_name,
        fset=_set_name,
        doc=u"""name of the material

        :Type: str
        """,
    )

    def _get_is_isotropic(self):
        """getter of is_isotropic"""
        return self._is_isotropic

    def _set_is_isotropic(self, value):
        """setter of is_isotropic"""
        check_var("is_isotropic", value, "bool")
        self._is_isotropic = value

    is_isotropic = property(
        fget=_get_is_isotropic,
        fset=_set_is_isotropic,
        doc=u"""If True, uniformity in all orientations

        :Type: bool
        """,
    )

    def _get_elec(self):
        """getter of elec"""
        return self._elec

    def _set_elec(self, value):
        """setter of elec"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "elec")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = MatElectrical()
        check_var("elec", value, "MatElectrical")
        self._elec = value

        if self._elec is not None:
            self._elec.parent = self

    elec = property(
        fget=_get_elec,
        fset=_set_elec,
        doc=u"""Electrical properties of the material

        :Type: MatElectrical
        """,
    )

    def _get_mag(self):
        """getter of mag"""
        return self._mag

    def _set_mag(self, value):
        """setter of mag"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "mag")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = MatMagnetics()
        check_var("mag", value, "MatMagnetics")
        self._mag = value

        if self._mag is not None:
            self._mag.parent = self

    mag = property(
        fget=_get_mag,
        fset=_set_mag,
        doc=u"""Magnetic properties of the material

        :Type: MatMagnetics
        """,
    )

    def _get_struct(self):
        """getter of struct"""
        return self._struct

    def _set_struct(self, value):
        """setter of struct"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "struct"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = MatStructural()
        check_var("struct", value, "MatStructural")
        self._struct = value

        if self._struct is not None:
            self._struct.parent = self

    struct = property(
        fget=_get_struct,
        fset=_set_struct,
        doc=u"""Structural properties of the material

        :Type: MatStructural
        """,
    )

    def _get_HT(self):
        """getter of HT"""
        return self._HT

    def _set_HT(self, value):
        """setter of HT"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "HT")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = MatHT()
        check_var("HT", value, "MatHT")
        self._HT = value

        if self._HT is not None:
            self._HT.parent = self

    HT = property(
        fget=_get_HT,
        fset=_set_HT,
        doc=u"""Heat Transfer properties of the material

        :Type: MatHT
        """,
    )

    def _get_eco(self):
        """getter of eco"""
        return self._eco

    def _set_eco(self, value):
        """setter of eco"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "eco")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = MatEconomical()
        check_var("eco", value, "MatEconomical")
        self._eco = value

        if self._eco is not None:
            self._eco.parent = self

    eco = property(
        fget=_get_eco,
        fset=_set_eco,
        doc=u"""Economical properties of the material

        :Type: MatEconomical
        """,
    )

    def _get_desc(self):
        """getter of desc"""
        return self._desc

    def _set_desc(self, value):
        """setter of desc"""
        check_var("desc", value, "str")
        self._desc = value

    desc = property(
        fget=_get_desc,
        fset=_set_desc,
        doc=u"""material description

        :Type: str
        """,
    )

    def _get_path(self):
        """getter of path"""
        return self._path

    def _set_path(self, value):
        """setter of path"""
        check_var("path", value, "str")
        self._path = value

    path = property(
        fget=_get_path,
        fset=_set_path,
        doc=u"""Path to the material file

        :Type: str
        """,
    )



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Import/ImportMatrixVal.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Import/ImportMatrixVal
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import set_array, check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .ImportMatrix import ImportMatrix

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Import.ImportMatrixVal.get_data import get_data
except ImportError as error:
    get_data = error


from numpy import array, array_equal
from ._check import InitUnKnowClassError


class ImportMatrixVal(ImportMatrix):
    """Import directly the value from the object"""

    VERSION = 1

    # cf Methods.Import.ImportMatrixVal.get_data
    if isinstance(get_data, ImportError):
        get_data = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use ImportMatrixVal method get_data: " + str(get_data)
                )
            )
        )
    else:
        get_data = get_data
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, value=None, is_transpose=False, init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "value" in list(init_dict.keys()):
                value = init_dict["value"]
            if "is_transpose" in list(init_dict.keys()):
                is_transpose = init_dict["is_transpose"]
        # Set the properties (value check and convertion are done in setter)
        self.value = value
        # Call ImportMatrix init
        super(ImportMatrixVal, self).__init__(is_transpose=is_transpose)
        # The class is frozen (in ImportMatrix init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        ImportMatrixVal_str = ""
        # Get the properties inherited from ImportMatrix
        ImportMatrixVal_str += super(ImportMatrixVal, self).__str__()
        ImportMatrixVal_str += (
            "value = "
            + linesep
            + str(self.value).replace(linesep, linesep + "\t")
            + linesep
            + linesep
        )
        return ImportMatrixVal_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from ImportMatrix
        if not super(ImportMatrixVal, self).__eq__(other):
            return False
        if not array_equal(other.value, self.value):
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from ImportMatrix
        diff_list.extend(super(ImportMatrixVal, self).compare(other, name=name))
        if not array_equal(other.value, self.value):
            diff_list.append(name + ".value")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from ImportMatrix
        S += super(ImportMatrixVal, self).__sizeof__()
        S += getsizeof(self.value)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from ImportMatrix
        ImportMatrixVal_dict = super(ImportMatrixVal, self).as_dict(**kwargs)
        if self.value is None:
            ImportMatrixVal_dict["value"] = None
        else:
            ImportMatrixVal_dict["value"] = self.value.tolist()
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        ImportMatrixVal_dict["__class__"] = "ImportMatrixVal"
        return ImportMatrixVal_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.value = None
        # Set to None the properties inherited from ImportMatrix
        super(ImportMatrixVal, self)._set_None()

    def _get_value(self):
        """getter of value"""
        return self._value

    def _set_value(self, value):
        """setter of value"""
        if type(value) is int and value == -1:
            value = array([])
        elif type(value) is list:
            try:
                value = array(value)
            except:
                pass
        check_var("value", value, "ndarray")
        self._value = value

    value = property(
        fget=_get_value,
        fset=_set_value,
        doc=u"""The matrix to return

        :Type: ndarray
        """,
    )



# -*-- coding: utf-8 -*
from numpy import exp


def rotate(self, angle):
    """Do the rotation of the PolarArc

    Parameters
    ----------
    self : PolarArc
        a PolarArc
    angle : float
        the angle for rotation [rad]

    Returns
    -------
    None
    """
    # check if the PolarArc is correct
    self.check()
    self.point_ref = self.point_ref * exp(1j * angle)



# -*- coding: utf-8 -*-

from matplotlib.lines import Line2D
from matplotlib.pyplot import axis, legend, plot, subplots, title
from numpy import array, linspace, meshgrid

from ....Functions.Winding.comp_wind_periodicity import comp_wind_periodicity
from ....Functions.Winding.gen_phase_list import gen_color, gen_name


def plot_winding(self, wind_mat=None, all_slot=False, is_show_fig=True):
    """Plot the Winding in a matplotlib fig

    Parameters
    ----------
    self : LamSlotWind
        A: LamSlotWind object
    wind_mat : numpy.ndarray
        Winding Matrix, if None will call comp_connection_mat (Default value = None)
    all_slot : bool
        True if we plot all slot and false when plotting only needed one(sym)
    is_show_fig : bool
        To call show at the end of the method
    Returns
    -------
    None
    """

    # We compute the wind_mat only if needed
    if wind_mat is None:
        wind_mat = self.winding.comp_connection_mat(self.slot.Zs)

    # Number of point on rad and tan direction
    Nrad, Ntan = self.winding.get_dim_wind()
    Zs = self.slot.Zs  # Number of slot

    # Number of Slot to plot
    if all_slot:  # Every Slot
        Nplot = Zs
    else:  # Only the needed one (sym)
        Nperw = comp_wind_periodicity(wind_mat)[0]  # Symmetry of the winding
        Nplot = Zs // Nperw

    qs = wind_mat.shape[3]  # Number of phase

    # Symbole for pole
    qs_color = gen_color(self.winding.qs)
    qs_name = gen_name(self.winding.qs)

    # Schematic slot without ratio
    Wt = 0.5
    W0 = 0.5
    H = 1

    # Coordinate of the First Slot   (center on 0)
    Slot_tan = array(
        [-Wt / 2 - W0 / 2, -W0 / 2, -W0 / 2, W0 / 2, W0 / 2, W0 / 2 + Wt / 2]
    )
    Slot_rad = [0, 0, H, H, 0, 0]

    # Duplicate the Slot along tan direction (angular abscissa )
    x = list()
    y = list()
    for i in range(0, Nplot):
        x.extend((Slot_tan + (Wt + W0) * i).tolist())
        y.extend(Slot_rad)

    # Plot the Schematics Slots
    fig, ax = subplots()
    plot(x, y, "r-")

    # First Winding Grid (Coordinate of the winding mark)
    range_x = linspace(-W0 / 2, W0 / 2, Ntan + 1, endpoint=False)
    range_y = linspace(0, H, Nrad + 1, endpoint=False)
    # We don't want the first and last point of the linespace
    Grid_x, Grid_y = meshgrid(range_x[1:], range_y[1:])

    # Plot the Winding Grid point by point by reading wind_mat
    for Zs in range(0, Nplot):  # For "every" Slot
        for q in range(0, qs):  # For every phase
            for r in range(0, Nrad):  # For every rad layer
                for theta in range(0, Ntan):  # For every tan layer
                    if wind_mat[r, theta, Zs, q] != 0:
                        # Add the correct mark at the correct coordinates
                        if wind_mat[r, theta, Zs, q] > 0:
                            plot(
                                Grid_x[r][theta] + Zs * (Wt + W0),
                                Grid_y[r][theta],
                                color=qs_color[q],
                                linewidth=0,
                                marker="+",
                                markeredgewidth=3,
                                markersize=20,
                            )
                        else:
                            plot(
                                Grid_x[r][theta] + Zs * (Wt + W0),
                                Grid_y[r][theta],
                                color=qs_color[q],
                                linewidth=0,
                                marker="x",
                                markeredgewidth=3,
                                markersize=20,
                            )
    if self.is_stator:
        Lam_Name = "Stator"
    else:
        Lam_Name = "Rotor"
    if all_slot or Nperw == 1:
        title(Lam_Name + "'s Winding (every slot)")
    else:
        title(Lam_Name + "'s Winding (periodicity 1/" + str(Nperw) + ")")

    axis("equal")
    ax.get_yaxis().set_visible(False)

    # Legend qs
    sym_leg = list()  # Symbol
    label_leg = list()  # Text
    for q in range(0, qs):  # Positive mark
        sym_leg.append(
            Line2D(
                [],
                [],
                color=qs_color[q],
                linewidth=0,
                marker="+",
                markeredgewidth=3,
                markersize=20,
            )
        )
        label_leg.append(qs_name[q] + "+")
    for q in range(0, qs):  # Negative mark
        sym_leg.append(
            Line2D(
                [],
                [],
                color=qs_color[q],
                linewidth=0,
                marker="x",
                markeredgewidth=3,
                markersize=20,
            )
        )
        label_leg.append(qs_name[q] + "-")

    legend(sym_leg, label_leg, ncol=2)

    if is_show_fig:
        fig.show()



from numpy import pi
from SciDataTool import Data1D, DataLinspace
from ....Methods.Simulation.Input import InputError


def comp_axes(self, machine, N0=None):
    """Compute simulation axes, i.e. space DataObject including (anti)-periodicity
    and time DataObject including (anti)-periodicity and accounting for rotating speed
    and number of revolutions

    Parameters
    ----------
    self : Input
        an Input object
    machine : Machine
        a Machine object
    N0 : float
        rotating speed [rpm]


    Returns
    -------
    Time : DataLinspace
        Time axis including (anti)-periodicity and accounting for rotating speed and number of revolutions
    Angle : DataLinspace
        Angle axis including (anti)-periodicity

    """
    if self.time is None and N0 is None:
        raise InputError("ERROR: time and N0 can't be both None")

    # Get machine pole pair number
    p = machine.get_pole_pair_number()

    # Get electrical fundamental frequency
    f_elec = self.comp_felec()

    # Airgap radius
    Rag = machine.comp_Rgap_mec()

    # Setup normalizations for time and angle axes
    norm_time = {
        "elec_order": f_elec,
        "mech_order": f_elec / p,
    }
    if N0 is not None:
        norm_time["angle_rotor"] = 1 / (360 * N0 / 60)

    norm_angle = {"space_order": p, "distance": 1 / Rag}

    # Create time axis
    if self.time is None:
        # Create time axis as a DataLinspace
        Time = DataLinspace(
            name="time",
            unit="s",
            initial=0,
            final=60 / N0 * self.Nrev,
            number=self.Nt_tot,
            include_endpoint=False,
            normalizations=norm_time,
        )
    else:
        # Load time data
        time = self.time.get_data()
        self.Nt_tot = len(time)
        Time = Data1D(name="time", unit="s", values=time, normalizations=norm_time)

    # Create angle axis
    if self.angle is None:
        # Create angle axis as a DataLinspace
        Angle = DataLinspace(
            name="angle",
            unit="rad",
            initial=0,
            final=2 * pi,
            number=self.Na_tot,
            include_endpoint=False,
            normalizations=norm_angle,
        )
    else:
        # Load angle data
        angle = self.angle.get_data()
        self.Na_tot = len(angle)
        Angle = Data1D(
            name="angle", unit="rad", values=angle, normalizations=norm_angle
        )

    return Time, Angle



# -*- coding: utf-8 -*-

from numpy import pi
from ....Classes.Lamination import Lamination


def comp_masses(self):
    """Compute the Lamination masses

    Parameters
    ----------
    self : LamHole
        A LamHole object

    Returns
    -------
    M_dict: dict
        Lamination mass dictionnary (Mtot, Mlam, Mmag) [kg]

    """

    M_dict = Lamination.comp_masses(self)

    Mmag = 0
    for hole in self.hole:
        if hole.has_magnet():
            Mmag += hole.Zh * hole.comp_mass_magnets()

    M_dict["Mmag"] = Mmag
    M_dict["Mtot"] += Mmag

    return M_dict



import random
from collections.abc import Iterable
from deap.tools import mutPolynomialBounded


def mutate(self, indiv):
    """Mutate the individual design variables with different strategies according to the variables types :
        - interval : Polynomial Bounded mutation or user defined
        - set : Resampling the variable according to its initialization function
        - pyleecan : Resampling the variable according to its initialization function

    Parameters
    ----------
    solver : Solver
        solver to perform the genetic algorithm with DEAP
    indiv : individual (e.g. OptiGenAlgIndivDeap)
        individual to mutate

    Returns
    -------
    is_mutation : boolean
        True if at least one mutation occured
    """
    is_mutation = False

    for k, design_var in enumerate(indiv.design_var):
        if self.p_mutate < random.random():  # Perform mutation
            is_mutation = True

            if self.mutator == None:
                if design_var.type_var == "interval":  # Interval variable
                    # Using polynomial bounded mutation as in Deb and al., "A Fast and Elitist Multiobjective Genetic Algorithm: NSGA-II"
                    if isinstance(indiv[k], Iterable):
                        indiv[k] = mutPolynomialBounded(
                            indiv[k], 20, *design_var.space, 1
                        )[0]
                    else:
                        # Function takes list in argument and returns list
                        indiv[k] = mutPolynomialBounded(
                            [indiv[k]], 20, *design_var.space, 1
                        )[0][0]
                else:  # Uniform mutation
                    indiv[k] = random.choice(design_var.space)
            else:  # User defined mutator
                if design_var.type_var == "interval":  # Interval variable
                    indiv[k] = self.mutator(indiv[k])
                else:
                    # TODO Allow to redefine mutators for set
                    indiv[k] = design_var.function(design_var.space)
    return is_mutation



from numpy import array, sum


def comp_point_ref(self, is_set=False):
    """Compute the point ref of the Surface

    Parameters
    ----------
    self : SurfRing
        A SurfRing object
    is_set: bool
        True to update the point_ref property

    Returns
    -------
    point_ref : complex
        the reference point of the surface
    """

    point_list = list()
    out_lines = self.out_surf.get_lines()
    in_lines = self.in_surf.get_lines()
    # Compute the ref point as the barycenter of half the lines
    for line in out_lines[: len(out_lines) / 2]:
        point_list.append(line.get_middle())
    for line in in_lines[: len(in_lines) / 2]:
        point_list.append(line.get_middle())
    point_ref = sum(array(point_list)) / len(point_list)

    if is_set:
        self.point_ref = point_ref
    return point_ref



# -*- coding: utf-8 -*-

import PySide2.QtCore
from PySide2.QtCore import Signal
from PySide2.QtWidgets import QWidget

from ......Classes.SlotW24 import SlotW24
from ......GUI import gui_option
from ......GUI.Dialog.DMachineSetup.SWSlot.PWSlot24.Gen_PWSlot24 import Gen_PWSlot24
from ......Methods.Slot.Slot import SlotCheckError

translate = PySide2.QtCore.QCoreApplication.translate


class PWSlot24(Gen_PWSlot24, QWidget):
    """Page to set the Slot Type 24"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for Slot combobox
    slot_name = "Slot Type 24"
    slot_type = SlotW24

    def __init__(self, lamination=None):
        """Initialize the GUI according to current lamination

        Parameters
        ----------
        self : PWSlot24
            A PWSlot24 widget
        lamination : Lamination
            current lamination to edit
        """
        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        self.lamination = lamination
        self.slot = lamination.slot
        # Set FloatEdit unit
        self.lf_W3.unit = "m"
        self.lf_H2.unit = "m"
        # Set unit name (m ou mm)
        wid_list = [self.unit_W3, self.unit_H2]
        for wid in wid_list:
            wid.setText("[" + gui_option.unit.get_m_name() + "]")

        # Fill the fields with the machine values (if they're filled)
        self.lf_W3.setValue(self.slot.W3)
        self.lf_H2.setValue(self.slot.H2)

        # Display the main output of the slot (surface, height...)
        self.w_out.comp_output()

        # Connect the signal/slot
        self.lf_W3.editingFinished.connect(self.set_W3)
        self.lf_H2.editingFinished.connect(self.set_H2)

    def set_W3(self):
        """Signal to update the value of W3 according to the line edit

        Parameters
        ----------
        self : PWSlot24
            A PWSlot24 object
        """
        self.slot.W3 = self.lf_W3.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_H2(self):
        """Signal to update the value of H2 according to the line edit

        Parameters
        ----------
        self : PWSlot24
            A PWSlot24 object
        """
        self.slot.H2 = self.lf_H2.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    @staticmethod
    def check(lam):
        """Check that the current lamination have all the needed field set

        Parameters
        ----------
        lam: LamSlotWind
            Lamination to check

        Returns
        -------
        error: str
            Error message (return None if no error)
        """

        # Check that everything is set
        if lam.slot.W3 is None:
            return "You must set W3 !"
        elif lam.slot.H2 is None:
            return "You must set H2 !"

        # Check that everything is set right
        # Constraints
        try:
            lam.slot.check()
        except SlotCheckError as error:
            return str(error)

        # Output
        try:
            yoke_height = lam.comp_height_yoke()
        except Exception as error:
            return "Unable to compute yoke height:" + str(error)
        if yoke_height <= 0:
            return "The slot height is greater than the lamination !"



from ....Methods.Slot.Slot import SlotCheckError


def check(self):
    """Check that the SlotUD is correctly defined"""

    if self.line_list is None:
        self.line_list = list()

    if len(self.line_list) == 0:
        raise SlotCheckError("SlotUD must have at least one line")

    for ii in range(len(self.line_list) - 1):
        if (
            abs(self.line_list[ii].get_end() - self.line_list[ii + 1].get_begin())
            > 1e-6
        ):
            raise SlotCheckError(
                "SlotUD line " + str(ii) + " and line " + str(ii + 1) + " doesn't match"
            )

    begin_id = self.wind_begin_index
    end_id = self.wind_end_index
    if begin_id is not None and end_id is not None:
        if begin_id > len(self.line_list):
            raise SlotCheckError(
                "SlotUD wind_begin_index is greater than line_list length"
            )
        if end_id > len(self.line_list):
            raise SlotCheckError(
                "SlotUD wind_end_index is greater than line_list length"
            )
        if begin_id > end_id:
            raise SlotCheckError(
                "SlotUD wind_begin_index is greater than wind_end_index"
            )



from numpy import cos, exp, sin
from ....Functions.Geometry.inter_line_circle import inter_line_circle
from ....Methods.Slot.HoleM53 import Slot53InterError


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : HoleM53
        A HoleM53 object

    Returns
    -------
    point_dict: dict
        A dict of the slot coordinates
    """
    Rext = self.get_Rext()

    Z7 = Rext - self.H0 - 1j * self.W1 / 2
    Z6 = Z7 - 1j * (self.H2 - self.H3) * cos(self.W4)
    Z8 = Z7 + (self.H2 - self.H3) * sin(self.W4)

    # Compute the coordinate in the ref of Z6 with rotation -W4
    Z5 = self.W2 * exp(-1j * self.W4) + Z6
    Z4 = (self.W2 - 1j * self.H3) * exp(-1j * self.W4) + Z6
    Z3 = (self.W2 + self.W3 - 1j * self.H3) * exp(-1j * self.W4) + Z6
    Z2 = (self.W2 + self.W3) * exp(-1j * self.W4) + Z6
    Z9 = (self.W2 + 1j * (self.H2 - self.H3)) * exp(-1j * self.W4) + Z6
    Z10 = (self.W2 + self.W3 + 1j * (self.H2 - self.H3)) * exp(-1j * self.W4) + Z6

    # Z1 and Z11 are defined as intersection between line and circle
    Zlist = inter_line_circle(Z8, Z10, Rext - self.H1)
    if len(Zlist) == 2 and Zlist[0].imag < 0 and Zlist[0].real > 0:
        Z11 = Zlist[0]
    elif len(Zlist) == 2 and Zlist[1].imag < 0 and Zlist[1].real > 0:
        Z11 = Zlist[1]
    else:
        raise Slot53InterError("ERROR: Slot 53, Can't find Z11 coordinates")

    Zlist = inter_line_circle(Z2, Z6, Rext - self.H1)
    if len(Zlist) == 2 and Zlist[0].imag < 0 and Zlist[0].real > 0:
        Z1 = Zlist[0]
    elif len(Zlist) == 2 and Zlist[1].imag < 0 and Zlist[1].real > 0:
        Z1 = Zlist[1]
    else:
        raise Slot53InterError("ERROR: Slot 53, Can't find Z1 coordinates")

    point_dict = dict()
    point_dict["Z1"] = Z1
    point_dict["Z2"] = Z2
    point_dict["Z3"] = Z3
    point_dict["Z4"] = Z4
    point_dict["Z5"] = Z5
    point_dict["Z6"] = Z6
    point_dict["Z7"] = Z7
    point_dict["Z8"] = Z8
    point_dict["Z9"] = Z9
    point_dict["Z10"] = Z10
    point_dict["Z11"] = Z11
    point_dict["Z1s"] = Z1.conjugate()
    point_dict["Z2s"] = Z2.conjugate()
    point_dict["Z3s"] = Z3.conjugate()
    point_dict["Z4s"] = Z4.conjugate()
    point_dict["Z5s"] = Z5.conjugate()
    point_dict["Z6s"] = Z6.conjugate()
    point_dict["Z7s"] = Z7.conjugate()
    point_dict["Z8s"] = Z8.conjugate()
    point_dict["Z9s"] = Z9.conjugate()
    point_dict["Z10s"] = Z10.conjugate()
    point_dict["Z11s"] = Z11.conjugate()
    return point_dict



# -*- coding: utf-8 -*-

from numpy import pi, arcsin, sin


def comp_surface(self):
    """Compute the Slot total surface (by analytical computation).
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW61
        A SlotW61 object

    Returns
    -------
    S: float
        Slot total surface [m**2]

    """

    [Z1, Z2, Z3, Z4, Z5, Z6, Z7, Z8, Z9, Z10] = self._comp_point_coordinate()
    Rext = self.get_Rbo()
    Rint = abs(Z5)

    # Surface of a slot pitch
    Sring = (pi * Rext ** 2 - pi * Rint ** 2) * 1 / self.Zs

    # Tooth surface
    St = self.H1 * self.W1 + self.H2 * self.W2 + self.H0 * (self.W0 + self.W1) / 2

    alpha_top = 2 * arcsin(self.W0 / (2 * Rext))
    Sarc_top = (Rext ** 2.0) / 2.0 * (alpha_top - sin(alpha_top))

    alpha_bot = 2 * arcsin(self.W2 / (2 * Rint))
    Sarc_bot = (Rint ** 2.0) / 2.0 * (alpha_bot - sin(alpha_bot))

    return Sring - (St + Sarc_top - Sarc_bot)



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Import/ImportGenMatrixSin.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Import/ImportGenMatrixSin
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .ImportMatrix import ImportMatrix

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Import.ImportGenMatrixSin.get_data import get_data
except ImportError as error:
    get_data = error

try:
    from ..Methods.Import.ImportGenMatrixSin.init_vector import init_vector
except ImportError as error:
    init_vector = error


from ._check import InitUnKnowClassError
from .ImportGenVectSin import ImportGenVectSin


class ImportGenMatrixSin(ImportMatrix):
    """To generate a Sinus matrix"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Import.ImportGenMatrixSin.get_data
    if isinstance(get_data, ImportError):
        get_data = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use ImportGenMatrixSin method get_data: " + str(get_data)
                )
            )
        )
    else:
        get_data = get_data
    # cf Methods.Import.ImportGenMatrixSin.init_vector
    if isinstance(init_vector, ImportError):
        init_vector = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use ImportGenMatrixSin method init_vector: "
                    + str(init_vector)
                )
            )
        )
    else:
        init_vector = init_vector
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, sin_list=-1, is_transpose=False, init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "sin_list" in list(init_dict.keys()):
                sin_list = init_dict["sin_list"]
            if "is_transpose" in list(init_dict.keys()):
                is_transpose = init_dict["is_transpose"]
        # Set the properties (value check and convertion are done in setter)
        self.sin_list = sin_list
        # Call ImportMatrix init
        super(ImportGenMatrixSin, self).__init__(is_transpose=is_transpose)
        # The class is frozen (in ImportMatrix init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        ImportGenMatrixSin_str = ""
        # Get the properties inherited from ImportMatrix
        ImportGenMatrixSin_str += super(ImportGenMatrixSin, self).__str__()
        if len(self.sin_list) == 0:
            ImportGenMatrixSin_str += "sin_list = []" + linesep
        for ii in range(len(self.sin_list)):
            tmp = self.sin_list[ii].__str__().replace(linesep, linesep + "\t") + linesep
            ImportGenMatrixSin_str += (
                "sin_list[" + str(ii) + "] =" + tmp + linesep + linesep
            )
        return ImportGenMatrixSin_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from ImportMatrix
        if not super(ImportGenMatrixSin, self).__eq__(other):
            return False
        if other.sin_list != self.sin_list:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from ImportMatrix
        diff_list.extend(super(ImportGenMatrixSin, self).compare(other, name=name))
        if (other.sin_list is None and self.sin_list is not None) or (
            other.sin_list is not None and self.sin_list is None
        ):
            diff_list.append(name + ".sin_list None mismatch")
        elif self.sin_list is None:
            pass
        elif len(other.sin_list) != len(self.sin_list):
            diff_list.append("len(" + name + ".sin_list)")
        else:
            for ii in range(len(other.sin_list)):
                diff_list.extend(
                    self.sin_list[ii].compare(
                        other.sin_list[ii], name=name + ".sin_list[" + str(ii) + "]"
                    )
                )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from ImportMatrix
        S += super(ImportGenMatrixSin, self).__sizeof__()
        if self.sin_list is not None:
            for value in self.sin_list:
                S += getsizeof(value)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from ImportMatrix
        ImportGenMatrixSin_dict = super(ImportGenMatrixSin, self).as_dict(**kwargs)
        if self.sin_list is None:
            ImportGenMatrixSin_dict["sin_list"] = None
        else:
            ImportGenMatrixSin_dict["sin_list"] = list()
            for obj in self.sin_list:
                if obj is not None:
                    ImportGenMatrixSin_dict["sin_list"].append(obj.as_dict(**kwargs))
                else:
                    ImportGenMatrixSin_dict["sin_list"].append(None)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        ImportGenMatrixSin_dict["__class__"] = "ImportGenMatrixSin"
        return ImportGenMatrixSin_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.sin_list = None
        # Set to None the properties inherited from ImportMatrix
        super(ImportGenMatrixSin, self)._set_None()

    def _get_sin_list(self):
        """getter of sin_list"""
        if self._sin_list is not None:
            for obj in self._sin_list:
                if obj is not None:
                    obj.parent = self
        return self._sin_list

    def _set_sin_list(self, value):
        """setter of sin_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "sin_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("sin_list", value, "[ImportGenVectSin]")
        self._sin_list = value

    sin_list = property(
        fget=_get_sin_list,
        fset=_set_sin_list,
        doc=u"""List of sinus vector to generate the matrix lines

        :Type: [ImportGenVectSin]
        """,
    )



# -*- coding: utf-8 -*-

from numpy import pi, sin, arcsin


def comp_surface_active(self):
    """Compute the Slot inner surface for winding (by analytical computation)

    Parameters
    ----------
    self : SlotW24
        A SlotW24 object

    Returns
    -------
    Swind: float
        Slot inner surface for winding [m**2]

    """

    Rbo = self.get_Rbo()
    point_dict = self._comp_point_coordinate()

    if self.is_outwards():
        Rint = Rbo
        Rext = abs(point_dict["Z2"])
    else:
        Rint = abs(point_dict["Z2"])
        Rext = Rbo

    # Surface of a slot pitch
    Sring = (pi * Rext ** 2 - pi * Rint ** 2) * 1 / self.Zs

    # Tooth surface
    St = self.H2 * self.W3

    alpha_bore = 2 * arcsin(self.W3 / (2 * Rint))
    Sarc_bore = (Rint ** 2.0) / 2.0 * (alpha_bore - sin(alpha_bore))

    alpha_yoke = 2 * arcsin(self.W3 / (2 * Rext))
    Sarc_yoke = (Rext ** 2.0) / 2.0 * (alpha_yoke - sin(alpha_yoke))

    if self.is_outwards():
        return Sring - (St + Sarc_yoke - Sarc_bore)
    else:
        return Sring - (St + Sarc_yoke - Sarc_bore)



# -*- coding: utf-8 -*-


def comp_parameters(self, output):
    """Compute the parameters dict for the equivalent electrical circuit:
    resistance, inductance and back electromotive force
    Parameters
    ----------
    self : EEC_PMSM
        an EEC_PMSM object
    output : Output
        an Output object
    """
    # TODO maybe set currents to small value if I is 0 to compute inductance

    PAR = self.parameters

    # Parameters to compute only once
    if "R20" not in PAR:
        PAR["R20"] = output.simu.machine.stator.comp_resistance_wind()
    if "phi" not in PAR:
        PAR["phi"] = self.fluxlink.comp_fluxlinkage(output)

    # Parameters which may vary for each simulation
    is_comp_ind = False
    # check for complete parameter set
    # (there may be some redundancy here but it seems simplier to implement)
    if not all(k in PAR for k in ("Phid", "Phiq", "Ld", "Lq")):
        is_comp_ind = True

    # check for d- and q-current (change)
    if "Id" not in PAR or PAR["Id"] != output.elec.Id_ref:
        PAR["Id"] = output.elec.Id_ref
        is_comp_ind = True

    if "Iq" not in PAR or PAR["Iq"] != output.elec.Iq_ref:
        PAR["Iq"] = output.elec.Iq_ref
        is_comp_ind = True

    # compute inductance if necessary
    if is_comp_ind:
        (phid, phiq) = self.indmag.comp_inductance(output)
        if PAR["Id"] != 0:
            PAR["Ld"] = (phid - PAR["phi"]) / PAR["Id"]
        else:
            PAR["Ld"] = None  # to have the parameters complete though

        if PAR["Iq"] != 0:
            PAR["Lq"] = phiq / PAR["Iq"]
        else:
            PAR["Lq"] = None  # to have the parameters complete though

        PAR["Phid"] = phid
        PAR["Phiq"] = phiq



from ...Classes.Lamination import Lamination
from ...Classes.Circle import Circle
from ...Functions.FEMM import is_eddies
from ...Functions.FEMM.assign_FEMM_surface import assign_FEMM_surface
from ...Functions.FEMM.comp_FEMM_dict import comp_FEMM_dict
from ...Functions.FEMM.create_FEMM_boundary_conditions import (
    create_FEMM_boundary_conditions,
)
from ...Functions.FEMM.create_FEMM_materials import create_FEMM_materials
from ...Functions.FEMM.get_sliding_band import get_sliding_band
from ...Functions.FEMM.get_airgap_surface import get_airgap_surface


def draw_FEMM(
    femm,
    output,
    is_mmfr,
    is_mmfs,
    sym,
    is_antiper,
    type_calc_leakage,
    is_remove_vent=False,
    is_remove_slotS=False,
    is_remove_slotR=False,
    type_BH_stator=0,
    type_BH_rotor=0,
    kgeo_fineness=1,
    kmesh_fineness=1,
    user_FEMM_dict={},
    path_save="FEMM_model.fem",
    is_sliding_band=True,
    transform_list=[],
    rotor_dxf=None,
    stator_dxf=None,
):
    """Draws and assigns the property of the machine in FEMM

    Parameters
    ----------
    femm : FEMMHandler
        client to send command to a FEMM instance
    output : Output
        Output object
    is_mmfr : bool
        1 to compute the rotor magnetomotive force / rotor
        magnetic field
    is_mmfs : bool
        1 to compute the stator magnetomotive force/stator
        magnetic field
    type_calc_leakage : int
        0 no leakage calculation
        1 calculation using single slot
    is_remove_vent : bool
        True to remove the ventilation ducts in FEMM (Default value = False)
    is_remove_slotS : bool
        True to solve without slot effect on the Stator (Default value = False)
    is_remove_slotR : bool
        True to solve without slot effect on the Rotor (Default value = False)
    type_BH_stator: int
        2 Infinite permeability, 1 to use linear B(H) curve according to mur_lin, 0 to use the B(H) curve
    type_BH_rotor: bool
        2 Infinite permeability, 1 to use linear B(H) curve according to mur_lin, 0 to use the B(H) curve
    kgeo_fineness : float
        global coefficient to adjust geometry fineness
        in FEMM (1: default ; > 1: finner ; < 1: less fine)
    kmesh_fineness : float
        global coefficient to adjust mesh fineness
        in FEMM (1: default ; > 1: finner ; < 1: less fine)
    sym : int
        the symmetry applied on the stator and the rotor (take into account antiperiodicity)
    is_antiper: bool
        To apply antiperiodicity boundary conditions
    rotor_dxf : DXFImport
        To use a dxf version of the rotor instead of build_geometry
    stator_dxf : DXFImport
        To use a dxf version of the stator instead of build_geometry

    Returns
    -------

    FEMM_dict : dict
        Dictionnary containing the main parameters of FEMM (including circuits and materials)
    """

    # Initialization from output for readibility
    BHs = output.geo.stator.BH_curve  # Stator B(H) curve
    BHr = output.geo.rotor.BH_curve  # Rotor B(H) curve
    Is = output.elec.Is  # Stator currents waveforms
    Ir = output.elec.Ir  # Rotor currents waveforms
    machine = output.simu.machine

    # Computing parameter (element size, arcspan...) needed to define the simulation
    FEMM_dict = comp_FEMM_dict(
        machine, kgeo_fineness, kmesh_fineness, type_calc_leakage
    )
    FEMM_dict.update(user_FEMM_dict)  # Overwrite some values if needed

    # The package must be initialized with the openfemm command.
    try:
        femm.openfemm(1)  # 1 == open in background, 0 == open normally
    except Exception as e:
        raise FEMMError(
            "ERROR: Unable to open FEMM, please check that FEMM is correctly installed\n"
            + str(e)
        )

    # We need to create a new Magnetostatics document to work on.
    femm.newdocument(0)

    # Minimize the main window for faster geometry creation.
    # femm.main_minimize()

    # defining the problem
    femm.mi_probdef(0, "meters", FEMM_dict["pbtype"], FEMM_dict["precision"])

    # Modifiy the machine to match the conditions
    machine = type(machine)(init_dict=machine.as_dict())
    if is_remove_slotR:  # Remove all slots on the rotor
        lam_dict = machine.rotor.as_dict()
        machine.rotor = Lamination(init_dict=lam_dict)
    if is_remove_slotS:  # Remove all slots on the stator
        lam_dict = machine.stator.as_dict()
        machine.stator = Lamination(init_dict=lam_dict)
    if is_remove_vent:  # Remove all ventilations
        machine.rotor.axial_vent = list()
        machine.stator.axial_vent = list()

    # Building geometry of the (modified) stator and the rotor
    surf_list = list()
    lam_list = machine.get_lam_list()
    lam_int = lam_list[0]
    lam_ext = lam_list[1]

    # Adding no_mesh for shaft if needed
    if lam_int.Rint > 0 and sym == 1:
        surf_list.append(Circle(point_ref=0, radius=lam_int.Rint, label="No_mesh"))

    # adding the Airgap surface
    if is_sliding_band:
        surf_list.extend(get_sliding_band(sym=sym, lam_int=lam_int, lam_ext=lam_ext))
    else:
        surf_list.extend(get_airgap_surface(lam_int=lam_int, lam_ext=lam_ext))

    # adding Both laminations surfaces (or import from DXF)
    if rotor_dxf is not None:
        femm.mi_readdxf(rotor_dxf.file_path)
        surf_list.extend(rotor_dxf.get_surfaces())
    else:
        surf_list.extend(machine.rotor.build_geometry(sym=sym))
    if stator_dxf is not None:
        femm.mi_readdxf(stator_dxf.file_path)
        surf_list.extend(stator_dxf.get_surfaces())
    else:
        surf_list.extend(machine.stator.build_geometry(sym=sym))

    # Applying user defined modifications
    for transform in transform_list:
        for surf in surf_list:
            if transform["label"] in surf.label and transform["type"] == "rotate":
                surf.rotate(transform["value"])
            elif transform["label"] in surf.label and transform["type"] == "translate":
                surf.translate(transform["value"])

    # Creation of all the materials and circuit in FEMM
    prop_dict, materials, circuits = create_FEMM_materials(
        femm,
        machine,
        surf_list,
        Is,
        Ir,
        BHs,
        BHr,
        is_mmfs,
        is_mmfr,
        type_BH_stator,
        type_BH_rotor,
        is_eddies,
        j_t0=0,
    )
    create_FEMM_boundary_conditions(femm=femm, sym=sym, is_antiper=is_antiper)

    # Draw and assign all the surfaces of the machine
    for surf in surf_list:
        label = surf.label
        # Get the correct element size and group according to the label
        surf.draw_FEMM(
            femm=femm,
            nodeprop="None",
            maxseg=FEMM_dict["arcspan"],  # max span of arc element in degrees
            propname="None",
            FEMM_dict=FEMM_dict,
            hide=False,
        )
        assign_FEMM_surface(
            femm, surf, prop_dict[label], FEMM_dict, machine.rotor, machine.stator
        )

    # Apply BC for DXF import
    if rotor_dxf is not None:
        for BC in rotor_dxf.BC_list:
            if BC[1] is True:  # Select Arc
                femm.mi_selectarcsegment(BC[0].real, BC[0].imag)
                femm.mi_setarcsegmentprop(FEMM_dict["arcspan"], BC[2], False, None)
            else:  # Select Line
                femm.mi_selectsegment(BC[0].real, BC[0].imag)
                femm.mi_setsegmentprop(BC[2], None, None, False, None)
            femm.mi_clearselected()

    # femm.mi_zoomnatural()  # Zoom out
    femm.mi_probdef(
        FEMM_dict["freqpb"],
        "meters",
        FEMM_dict["pbtype"],
        FEMM_dict["precision"],
        FEMM_dict["Lfemm"],
        FEMM_dict["minangle"],
        FEMM_dict["acsolver"],
    )
    femm.mi_smartmesh(FEMM_dict["smart_mesh"])
    femm.mi_saveas(path_save)  # Save
    FEMM_dict["path_save"] = path_save
    # femm.mi_close()

    FEMM_dict["materials"] = materials
    FEMM_dict["circuits"] = circuits

    return FEMM_dict


class FEMMError(Exception):
    """Raised when FEMM is not possible to run"""

    pass



from ...Classes.Material import Material
from ...Functions.Material.compare_material import compare_material


def replace_material_pyleecan_obj(obj, mat1, mat2, comp_name_path=True):
    """
    replace first material by the second in the object

    Parameters
    ----------
    obj: Pyleecan object
    mat1: Material
        material to replace
    mat2: Material
        new material
    comp_name_path: bool
        replace strictly mat1 or replace materials without comparing mat1.path and mat1.name

    Returns
    -------
    is_change: bool
        True if a material has been replaced
    """
    is_change = False
    obj_dict = obj.as_dict()
    if comp_name_path:
        for key, val in obj_dict.items():
            if isinstance(getattr(obj, key), Material) and getattr(obj, key) == mat1:
                setattr(obj, key, mat2)
                is_change = True
            # Call the function recursively to modify attributes materials
            elif isinstance(val, dict):
                is_change_recurs = replace_material_pyleecan_obj(
                    getattr(obj, key), mat1, mat2, comp_name_path
                )
                # update is_change if needed
                if not is_change:
                    is_change = is_change_recurs
    else:
        for key, val in obj_dict.items():
            # Compare materials with mat1 without name and path
            if isinstance(getattr(obj, key), Material) and compare_material(
                getattr(obj, key), mat1
            ):
                setattr(obj, key, mat2)
                is_change = True
            # Call the function recursively to modify attributes materials
            elif isinstance(val, dict):
                is_change_recurs = replace_material_pyleecan_obj(
                    getattr(obj, key), mat1, mat2, comp_name_path
                )
                # update is_change if needed
                if not is_change:
                    is_change = is_change_recurs

    return is_change



# -*- coding: utf-8 -*-

from ....Classes.ImportGenVectSin import ImportGenVectSin
from ....Methods.Import.ImportGenMatrixSin import InitSinMatDimError


def init_vector(self, f, A, Phi, N, Tf):
    """Create the sin_list according to the input list

    Parameters
    ----------
    self : ImportGenMatrixSin
        An ImportGenMatrixSin object
    f: list
        list of frequency for the sinus
    A: list
        list of amplitude for the sinus
    Phi: list
        list of phase for the sinus
    N: int
        Number of point (same for all the sinus)
    Tf: float
        End time (same for all the sinus)
    """

    if len(f) != len(A):
        raise InitSinMatDimError("ERROR: f and A have different dimensions")
    if len(f) != len(Phi):
        raise InitSinMatDimError("ERROR: f and Phi have different dimensions")

    self.sin_list = list()
    for ii in range(len(f)):
        self.sin_list.append(
            ImportGenVectSin(f=f[ii], A=A[ii], Phi=Phi[ii], N=N, Tf=Tf)
        )



# -*- coding: utf-8 -*-

# File generated according to WImportMatrixTable.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_WImportMatrixTable(object):
    def setupUi(self, WImportMatrixTable):
        if not WImportMatrixTable.objectName():
            WImportMatrixTable.setObjectName(u"WImportMatrixTable")
        WImportMatrixTable.resize(546, 511)
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            WImportMatrixTable.sizePolicy().hasHeightForWidth()
        )
        WImportMatrixTable.setSizePolicy(sizePolicy)
        WImportMatrixTable.setMinimumSize(QSize(0, 0))
        self.horizontalLayout = QHBoxLayout(WImportMatrixTable)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.in_matrix = QLabel(WImportMatrixTable)
        self.in_matrix.setObjectName(u"in_matrix")

        self.horizontalLayout.addWidget(self.in_matrix)

        self.b_convert = QPushButton(WImportMatrixTable)
        self.b_convert.setObjectName(u"b_convert")

        self.horizontalLayout.addWidget(self.b_convert)

        self.b_tab = QPushButton(WImportMatrixTable)
        self.b_tab.setObjectName(u"b_tab")

        self.horizontalLayout.addWidget(self.b_tab)

        self.b_plot = QPushButton(WImportMatrixTable)
        self.b_plot.setObjectName(u"b_plot")

        self.horizontalLayout.addWidget(self.b_plot)

        self.retranslateUi(WImportMatrixTable)

        QMetaObject.connectSlotsByName(WImportMatrixTable)

    # setupUi

    def retranslateUi(self, WImportMatrixTable):
        WImportMatrixTable.setWindowTitle(
            QCoreApplication.translate("WImportMatrixTable", u"Form", None)
        )
        self.in_matrix.setText(
            QCoreApplication.translate(
                "WImportMatrixTable", u"Matrix size (100,2)", None
            )
        )
        self.b_convert.setText(
            QCoreApplication.translate("WImportMatrixTable", u"Convert to Excel", None)
        )
        self.b_tab.setText(
            QCoreApplication.translate("WImportMatrixTable", u"Edit Table", None)
        )
        self.b_plot.setText(
            QCoreApplication.translate("WImportMatrixTable", u"Preview Plot", None)
        )

    # retranslateUi



from numpy import array


class XOutputError(Exception):
    pass


def _get_symbol_data_(self, symbol, index):
    """Helper function to get data and label by symbol to avoid duplicate code."""
    # Get the data
    if symbol in self.keys():  # DataKeeper or OptiObjective
        data = self[symbol]
        values = array(data.result)[index]
    elif symbol in [pe.symbol for pe in self.paramexplorer_list]:  # ParamSetter
        data = self.get_paramexplorer(symbol)
        values = array(data.value)[index]
    else:  # ParamSetter
        symbol_ = next(iter(self.keys()))
        self.get_logger().warning(
            f"XOutput.plot_pareto(): Symbol '{symbol}' not found. "
            + f"Using symbol '{symbol_}' instead."
        )
        symbol = symbol_
        data = self[symbol]
        values = array(data.result)[index]

    # label definition
    label = symbol
    if data.unit not in ["", None]:
        label += " [{}]".format(data.unit)

    return values, label



# -*- coding: utf-8 -*-

from numpy import arctan, exp, sqrt, tan


def comp_flower_arc(alpha, Rarc, Rext):
    """Compute the angle and end of a "flower arc".

    Parameters
    ----------
    alpha : float
        Angular width of the arc (angle(z_top_feft)=alpha/2)
    Rarc : float
        The radius of the arc to draw [m]
    Rext : float
        The radius of the external arc ((Rext,0) is on the
        flower arc) [m]

    Returns
    -------
    (alpha_lim,z_top_left,z_top_right) : (float, complex, complex)
        alpha_lim: angle of the arc [ard]
        z_top_left: first point of the arc
        z_top_right: second point fo the arc

    """
    # For readibility
    R = Rarc
    T = tan(alpha / 2)

    # Computation of the angle of the Arc (analitical solution for
    # equation : tan(alpha_mag/2) = Rsin(alpha)/(Rext-R(1-cos(alpha))
    alpha_lim1 = -2 * arctan(
        (R + sqrt(R ** 2 + 2 * R * Rext * T ** 2 - Rext ** 2 * T ** 2))
        / (T * (2 * (R - Rext)))
    )
    alpha_lim2 = -2 * arctan(
        (R - sqrt(R ** 2 + 2 * R * Rext * T ** 2 - Rext ** 2 * T ** 2))
        / (T * (2 * R - Rext))
    )
    alpha_lim = min(abs(alpha_lim1), abs(alpha_lim2))

    z_top_left = R * exp(+1j * alpha_lim) + (Rext - R)
    z_top_right = R * exp(-1j * alpha_lim) + (Rext - R)

    return (alpha_lim, z_top_left, z_top_right)



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Optimization/OptiGenAlg.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Optimization/OptiGenAlg
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .OptiSolver import OptiSolver

from ntpath import basename
from os.path import isfile
from ._check import CheckTypeError
import numpy as np
import random
from ._check import InitUnKnowClassError
from .OptiProblem import OptiProblem
from .XOutput import XOutput


class OptiGenAlg(OptiSolver):
    """Genetic algorithm class"""

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        selector=None,
        crossover=None,
        mutator=None,
        p_cross=0.9,
        p_mutate=0.1,
        size_pop=40,
        nb_gen=100,
        problem=-1,
        xoutput=-1,
        logger_name="Pyleecan.OptiSolver",
        is_keep_all_output=False,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "selector" in list(init_dict.keys()):
                selector = init_dict["selector"]
            if "crossover" in list(init_dict.keys()):
                crossover = init_dict["crossover"]
            if "mutator" in list(init_dict.keys()):
                mutator = init_dict["mutator"]
            if "p_cross" in list(init_dict.keys()):
                p_cross = init_dict["p_cross"]
            if "p_mutate" in list(init_dict.keys()):
                p_mutate = init_dict["p_mutate"]
            if "size_pop" in list(init_dict.keys()):
                size_pop = init_dict["size_pop"]
            if "nb_gen" in list(init_dict.keys()):
                nb_gen = init_dict["nb_gen"]
            if "problem" in list(init_dict.keys()):
                problem = init_dict["problem"]
            if "xoutput" in list(init_dict.keys()):
                xoutput = init_dict["xoutput"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
            if "is_keep_all_output" in list(init_dict.keys()):
                is_keep_all_output = init_dict["is_keep_all_output"]
        # Set the properties (value check and convertion are done in setter)
        self.selector = selector
        self.crossover = crossover
        self.mutator = mutator
        self.p_cross = p_cross
        self.p_mutate = p_mutate
        self.size_pop = size_pop
        self.nb_gen = nb_gen
        # Call OptiSolver init
        super(OptiGenAlg, self).__init__(
            problem=problem,
            xoutput=xoutput,
            logger_name=logger_name,
            is_keep_all_output=is_keep_all_output,
        )
        # The class is frozen (in OptiSolver init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        OptiGenAlg_str = ""
        # Get the properties inherited from OptiSolver
        OptiGenAlg_str += super(OptiGenAlg, self).__str__()
        if self._selector_str is not None:
            OptiGenAlg_str += "selector = " + self._selector_str + linesep
        elif self._selector_func is not None:
            OptiGenAlg_str += "selector = " + str(self._selector_func) + linesep
        else:
            OptiGenAlg_str += "selector = None" + linesep + linesep
        if self._crossover_str is not None:
            OptiGenAlg_str += "crossover = " + self._crossover_str + linesep
        elif self._crossover_func is not None:
            OptiGenAlg_str += "crossover = " + str(self._crossover_func) + linesep
        else:
            OptiGenAlg_str += "crossover = None" + linesep + linesep
        if self._mutator_str is not None:
            OptiGenAlg_str += "mutator = " + self._mutator_str + linesep
        elif self._mutator_func is not None:
            OptiGenAlg_str += "mutator = " + str(self._mutator_func) + linesep
        else:
            OptiGenAlg_str += "mutator = None" + linesep + linesep
        OptiGenAlg_str += "p_cross = " + str(self.p_cross) + linesep
        OptiGenAlg_str += "p_mutate = " + str(self.p_mutate) + linesep
        OptiGenAlg_str += "size_pop = " + str(self.size_pop) + linesep
        OptiGenAlg_str += "nb_gen = " + str(self.nb_gen) + linesep
        return OptiGenAlg_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from OptiSolver
        if not super(OptiGenAlg, self).__eq__(other):
            return False
        if other._selector_str != self._selector_str:
            return False
        if other._crossover_str != self._crossover_str:
            return False
        if other._mutator_str != self._mutator_str:
            return False
        if other.p_cross != self.p_cross:
            return False
        if other.p_mutate != self.p_mutate:
            return False
        if other.size_pop != self.size_pop:
            return False
        if other.nb_gen != self.nb_gen:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from OptiSolver
        diff_list.extend(super(OptiGenAlg, self).compare(other, name=name))
        if other._selector_str != self._selector_str:
            diff_list.append(name + ".selector")
        if other._crossover_str != self._crossover_str:
            diff_list.append(name + ".crossover")
        if other._mutator_str != self._mutator_str:
            diff_list.append(name + ".mutator")
        if other._p_cross != self._p_cross:
            diff_list.append(name + ".p_cross")
        if other._p_mutate != self._p_mutate:
            diff_list.append(name + ".p_mutate")
        if other._size_pop != self._size_pop:
            diff_list.append(name + ".size_pop")
        if other._nb_gen != self._nb_gen:
            diff_list.append(name + ".nb_gen")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from OptiSolver
        S += super(OptiGenAlg, self).__sizeof__()
        S += getsizeof(self._selector_str)
        S += getsizeof(self._crossover_str)
        S += getsizeof(self._mutator_str)
        S += getsizeof(self.p_cross)
        S += getsizeof(self.p_mutate)
        S += getsizeof(self.size_pop)
        S += getsizeof(self.nb_gen)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from OptiSolver
        OptiGenAlg_dict = super(OptiGenAlg, self).as_dict(**kwargs)
        if self._selector_str is not None:
            OptiGenAlg_dict["selector"] = self._selector_str
        elif "keep_function" in kwargs and kwargs["keep_function"]:
            OptiGenAlg_dict["selector"] = self.selector
        else:
            OptiGenAlg_dict["selector"] = None
            if self.selector is not None:
                self.get_logger().warning(
                    "OptiGenAlg.as_dict(): "
                    + f"Function {self.selector.__name__} is not serializable "
                    + "and will be converted to None."
                )
        if self._crossover_str is not None:
            OptiGenAlg_dict["crossover"] = self._crossover_str
        elif "keep_function" in kwargs and kwargs["keep_function"]:
            OptiGenAlg_dict["crossover"] = self.crossover
        else:
            OptiGenAlg_dict["crossover"] = None
            if self.crossover is not None:
                self.get_logger().warning(
                    "OptiGenAlg.as_dict(): "
                    + f"Function {self.crossover.__name__} is not serializable "
                    + "and will be converted to None."
                )
        if self._mutator_str is not None:
            OptiGenAlg_dict["mutator"] = self._mutator_str
        elif "keep_function" in kwargs and kwargs["keep_function"]:
            OptiGenAlg_dict["mutator"] = self.mutator
        else:
            OptiGenAlg_dict["mutator"] = None
            if self.mutator is not None:
                self.get_logger().warning(
                    "OptiGenAlg.as_dict(): "
                    + f"Function {self.mutator.__name__} is not serializable "
                    + "and will be converted to None."
                )
        OptiGenAlg_dict["p_cross"] = self.p_cross
        OptiGenAlg_dict["p_mutate"] = self.p_mutate
        OptiGenAlg_dict["size_pop"] = self.size_pop
        OptiGenAlg_dict["nb_gen"] = self.nb_gen
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        OptiGenAlg_dict["__class__"] = "OptiGenAlg"
        return OptiGenAlg_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.selector = None
        self.crossover = None
        self.mutator = None
        self.p_cross = None
        self.p_mutate = None
        self.size_pop = None
        self.nb_gen = None
        # Set to None the properties inherited from OptiSolver
        super(OptiGenAlg, self)._set_None()

    def _get_selector(self):
        """getter of selector"""
        return self._selector_func

    def _set_selector(self, value):
        """setter of selector"""
        if value is None:
            self._selector_str = None
            self._selector_func = None
        elif isinstance(value, str) and "lambda" in value:
            self._selector_str = value
            self._selector_func = eval(value)
        elif isinstance(value, str) and isfile(value) and value[-3:] == ".py":
            self._selector_str = value
            f = open(value, "r")
            exec(f.read(), globals())
            self._selector_func = eval(basename(value[:-3]))
        elif callable(value):
            self._selector_str = None
            self._selector_func = value
        else:
            raise CheckTypeError(
                "For property selector Expected function or str (path to python file or lambda), got: "
                + str(type(value))
            )

    selector = property(
        fget=_get_selector,
        fset=_set_selector,
        doc="""Selector of the genetic algorithm

        :Type: function
        """,
    )

    def _get_crossover(self):
        """getter of crossover"""
        return self._crossover_func

    def _set_crossover(self, value):
        """setter of crossover"""
        if value is None:
            self._crossover_str = None
            self._crossover_func = None
        elif isinstance(value, str) and "lambda" in value:
            self._crossover_str = value
            self._crossover_func = eval(value)
        elif isinstance(value, str) and isfile(value) and value[-3:] == ".py":
            self._crossover_str = value
            f = open(value, "r")
            exec(f.read(), globals())
            self._crossover_func = eval(basename(value[:-3]))
        elif callable(value):
            self._crossover_str = None
            self._crossover_func = value
        else:
            raise CheckTypeError(
                "For property crossover Expected function or str (path to python file or lambda), got: "
                + str(type(value))
            )

    crossover = property(
        fget=_get_crossover,
        fset=_set_crossover,
        doc="""Crossover of the genetic algorithm

        :Type: function
        """,
    )

    def _get_mutator(self):
        """getter of mutator"""
        return self._mutator_func

    def _set_mutator(self, value):
        """setter of mutator"""
        if value is None:
            self._mutator_str = None
            self._mutator_func = None
        elif isinstance(value, str) and "lambda" in value:
            self._mutator_str = value
            self._mutator_func = eval(value)
        elif isinstance(value, str) and isfile(value) and value[-3:] == ".py":
            self._mutator_str = value
            f = open(value, "r")
            exec(f.read(), globals())
            self._mutator_func = eval(basename(value[:-3]))
        elif callable(value):
            self._mutator_str = None
            self._mutator_func = value
        else:
            raise CheckTypeError(
                "For property mutator Expected function or str (path to python file or lambda), got: "
                + str(type(value))
            )

    mutator = property(
        fget=_get_mutator,
        fset=_set_mutator,
        doc="""Mutator of the genetic algorithm

        :Type: function
        """,
    )

    def _get_p_cross(self):
        """getter of p_cross"""
        return self._p_cross

    def _set_p_cross(self, value):
        """setter of p_cross"""
        check_var("p_cross", value, "float", Vmin=0, Vmax=1)
        self._p_cross = value

    p_cross = property(
        fget=_get_p_cross,
        fset=_set_p_cross,
        doc="""Probability of crossover

        :Type: float
        :min: 0
        :max: 1
        """,
    )

    def _get_p_mutate(self):
        """getter of p_mutate"""
        return self._p_mutate

    def _set_p_mutate(self, value):
        """setter of p_mutate"""
        check_var("p_mutate", value, "float", Vmin=0, Vmax=1)
        self._p_mutate = value

    p_mutate = property(
        fget=_get_p_mutate,
        fset=_set_p_mutate,
        doc="""Probability of mutation 

        :Type: float
        :min: 0
        :max: 1
        """,
    )

    def _get_size_pop(self):
        """getter of size_pop"""
        return self._size_pop

    def _set_size_pop(self, value):
        """setter of size_pop"""
        check_var("size_pop", value, "int", Vmin=1)
        self._size_pop = value

    size_pop = property(
        fget=_get_size_pop,
        fset=_set_size_pop,
        doc="""Size of the population

        :Type: int
        :min: 1
        """,
    )

    def _get_nb_gen(self):
        """getter of nb_gen"""
        return self._nb_gen

    def _set_nb_gen(self, value):
        """setter of nb_gen"""
        check_var("nb_gen", value, "int", Vmin=1)
        self._nb_gen = value

    nb_gen = property(
        fget=_get_nb_gen,
        fset=_set_nb_gen,
        doc="""Number of generations

        :Type: int
        :min: 1
        """,
    )
